@@ -1,3 +1,18 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.benchmark;
 
 import javassist.util.proxy.MethodFilter;
@@ -11,10 +26,25 @@ import net.bytebuddy.dynamic.scaffold.TypeValidation;
 import net.bytebuddy.implementation.Implementation;
 import net.bytebuddy.implementation.MethodDelegation;
 import net.bytebuddy.implementation.SuperMethodCall;
-import net.bytebuddy.implementation.bind.annotation.*;
+import net.bytebuddy.implementation.bind.annotation.AllArguments;
+import net.bytebuddy.implementation.bind.annotation.RuntimeType;
+import net.bytebuddy.implementation.bind.annotation.SuperCall;
+import net.bytebuddy.implementation.bind.annotation.SuperMethod;
+import net.bytebuddy.implementation.bind.annotation.This;
 import net.bytebuddy.pool.TypePool;
-import net.sf.cglib.proxy.*;
-import org.openjdk.jmh.annotations.*;
+import net.bytebuddy.utility.nullability.MaybeNull;
+import net.sf.cglib.proxy.CallbackHelper;
+import net.sf.cglib.proxy.Enhancer;
+import net.sf.cglib.proxy.MethodInterceptor;
+import net.sf.cglib.proxy.MethodProxy;
+import net.sf.cglib.proxy.NoOp;
+import org.openjdk.jmh.annotations.Benchmark;
+import org.openjdk.jmh.annotations.BenchmarkMode;
+import org.openjdk.jmh.annotations.Mode;
+import org.openjdk.jmh.annotations.OutputTimeUnit;
+import org.openjdk.jmh.annotations.Scope;
+import org.openjdk.jmh.annotations.Setup;
+import org.openjdk.jmh.annotations.State;
 
 import java.lang.reflect.Method;
 import java.net.URL;
@@ -71,51 +101,61 @@ public class ClassByExtensionBenchmark {
     /**
      * An implementation to be used by {@link ClassByExtensionBenchmark#benchmarkByteBuddyWithProxyAndReusedDelegator()}.
      */
+    @MaybeNull
     private Implementation proxyInterceptor;
 
     /**
      * An implementation to be used by {@link ClassByExtensionBenchmark#benchmarkByteBuddyWithAccessorAndReusedDelegator()}.
      */
+    @MaybeNull
     private Implementation accessInterceptor;
 
     /**
      * An implementation to be used by {@link ClassByExtensionBenchmark#benchmarkByteBuddyWithPrefixAndReusedDelegator()}.
      */
+    @MaybeNull
     private Implementation.Composable prefixInterceptor;
 
     /**
      * A description of {@link ClassByExtensionBenchmark#baseClass}.
      */
+    @MaybeNull
     private TypeDescription baseClassDescription;
 
     /**
      * A description of {@link ByteBuddyProxyInterceptor}.
      */
+    @MaybeNull
     private TypeDescription proxyClassDescription;
 
     /**
      * A description of {@link ByteBuddyAccessInterceptor}.
      */
+    @MaybeNull
     private TypeDescription accessClassDescription;
 
     /**
      * A description of {@link ByteBuddyPrefixInterceptor}.
      */
+    @MaybeNull
     private TypeDescription prefixClassDescription;
 
     /**
      * A method delegation to {@link ByteBuddyProxyInterceptor}.
      */
+    @MaybeNull
     private Implementation proxyInterceptorDescription;
 
     /**
      * A method delegation to {@link ByteBuddyAccessInterceptor}.
      */
+    @MaybeNull
     private Implementation accessInterceptorDescription;
 
     /**
      * A method delegation to {@link ByteBuddyPrefixInterceptor}.
      */
+    @MaybeNull
     private Implementation.Composable prefixInterceptorDescription;
 
     /**
@@ -126,10 +166,10 @@ public class ClassByExtensionBenchmark {
         proxyInterceptor = MethodDelegation.to(ByteBuddyProxyInterceptor.class);
         accessInterceptor = MethodDelegation.to(ByteBuddyAccessInterceptor.class);
         prefixInterceptor = MethodDelegation.to(ByteBuddyPrefixInterceptor.class);
-        baseClassDescription = TypePool.Default.ofClassPath().describe(baseClass.getName()).resolve();
-        proxyClassDescription = TypePool.Default.ofClassPath().describe(ByteBuddyProxyInterceptor.class.getName()).resolve();
-        accessClassDescription = TypePool.Default.ofClassPath().describe(ByteBuddyAccessInterceptor.class.getName()).resolve();
-        prefixClassDescription = TypePool.Default.ofClassPath().describe(ByteBuddyPrefixInterceptor.class.getName()).resolve();
+        baseClassDescription = TypePool.Default.ofSystemLoader().describe(baseClass.getName()).resolve();
+        proxyClassDescription = TypePool.Default.ofSystemLoader().describe(ByteBuddyProxyInterceptor.class.getName()).resolve();
+        accessClassDescription = TypePool.Default.ofSystemLoader().describe(ByteBuddyAccessInterceptor.class.getName()).resolve();
+        prefixClassDescription = TypePool.Default.ofSystemLoader().describe(ByteBuddyPrefixInterceptor.class.getName()).resolve();
         proxyInterceptorDescription = MethodDelegation.to(proxyClassDescription);
         accessInterceptorDescription = MethodDelegation.to(accessClassDescription);
         prefixInterceptorDescription = MethodDelegation.to(prefixClassDescription);
@@ -438,12 +478,10 @@ public class ClassByExtensionBenchmark {
         enhancer.setInterceptDuringConstruction(true);
         enhancer.setClassLoader(newClassLoader());
         enhancer.setSuperclass(baseClass);
-        CallbackHelper callbackHelper = new CallbackHelper(baseClass, new Class[0]) {
-            @Override
+        CallbackHelper callbackHelper = new CallbackHelper(baseClass, new Class<?>[0]) {
             protected Object getCallback(Method method) {
                 if (method.getDeclaringClass() == baseClass) {
                     return new MethodInterceptor() {
-                        @Override
                         public Object intercept(Object object,
                                                 Method method,
                                                 Object[] arguments,
@@ -470,7 +508,6 @@ public class ClassByExtensionBenchmark {
     @Benchmark
     public ExampleClass benchmarkJavassist() throws Exception {
         ProxyFactory proxyFactory = new ProxyFactory() {
-            @Override
             protected ClassLoader getClassLoader() {
                 return newClassLoader();
             }
@@ -489,8 +526,8 @@ public class ClassByExtensionBenchmark {
             public Object invoke(Object self,
                                  Method thisMethod,
                                  Method proceed,
-                                 Object[] args) throws Throwable {
-                return proceed.invoke(self, args);
+                                 Object[] argument) throws Throwable {
+                return proceed.invoke(self, argument);
             }
         });
         return (ExampleClass) instance;
