@@ -15,7 +15,22 @@
  */
 package net.bytebuddy.description;
 
-import net.bytebuddy.description.modifier.*;
+import net.bytebuddy.description.modifier.EnumerationState;
+import net.bytebuddy.description.modifier.FieldManifestation;
+import net.bytebuddy.description.modifier.FieldPersistence;
+import net.bytebuddy.description.modifier.Mandate;
+import net.bytebuddy.description.modifier.MethodManifestation;
+import net.bytebuddy.description.modifier.MethodStrictness;
+import net.bytebuddy.description.modifier.Openness;
+import net.bytebuddy.description.modifier.Ownership;
+import net.bytebuddy.description.modifier.ParameterManifestation;
+import net.bytebuddy.description.modifier.ProvisioningState;
+import net.bytebuddy.description.modifier.RequiredPhase;
+import net.bytebuddy.description.modifier.SynchronizationState;
+import net.bytebuddy.description.modifier.SyntheticState;
+import net.bytebuddy.description.modifier.Transitivity;
+import net.bytebuddy.description.modifier.TypeManifestation;
+import net.bytebuddy.description.modifier.Visibility;
 import org.objectweb.asm.Opcodes;
 
 /**
@@ -153,6 +168,26 @@ public interface ModifierReviewable {
         EnumerationState getEnumerationState();
     }
 
+    /**
+     * A modifier reviewable for elements that can be mandated.
+     */
+    interface OfMandatable extends ModifierReviewable {
+
+        /**
+         * Specifies if the modifier described by this object is mandated.
+         *
+         * @return {@code true} if the modifier described by this object is mandated.
+         */
+        boolean isMandated();
+
+        /**
+         * Returns the mandate of this instance.
+         *
+         * @return The mandate of this instance.
+         */
+        Mandate getMandate();
+    }
+
     /**
      * A modifier reviewable for a {@link net.bytebuddy.description.type.TypeDescription}.
      */
@@ -279,14 +314,7 @@ public interface ModifierReviewable {
     /**
      * A modifier reviewable for a {@link net.bytebuddy.description.method.ParameterDescription}.
      */
-    interface ForParameterDescription extends ModifierReviewable {
-
-        /**
-         * CSpecifies if the modifier described by this object is mandated.
-         *
-         * @return {@code true} if the modifier described by this object is mandated.
-         */
-        boolean isMandated();
+    interface ForParameterDescription extends OfMandatable {
 
         /**
          * Returns this parameter's manifestation.
@@ -303,10 +331,69 @@ public interface ModifierReviewable {
         ProvisioningState getProvisioningState();
     }
 
+    /**
+     * Declares modifiers as they can be observed on a Java module.
+     */
+    interface ForModuleDescription extends ModifierReviewable {
+
+        /**
+         * Returns {@code true} if the module is open.
+         *
+         * @return {@code true} if the module is open.
+         */
+        boolean isOpen();
+
+        /**
+         * Returns the openness of this module.
+         *
+         * @return A description of the openness of this module.
+         */
+        Openness getOpenness();
+    }
+
+    /**
+     * Declares modifiers as they can be observed on a module requirement declaration.
+     */
+    interface ForModuleRequirement extends OfMandatable {
+
+        /**
+         * Returns {@code true} if the module requirement is transitive.
+         *
+         * @return {@code true} if the module requirement is transitive.
+         */
+        boolean isTransitive();
+
+        /**
+         * Returns {@code true} if the module requirement is during the static phase only.
+         *
+         * @return {@code true} if the module requirement is during the static phase only.
+         */
+        boolean isStaticPhase();
+
+        /**
+         * Returns the transitivity of this module requirement.
+         *
+         * @return The transitivity of this module requirement.
+         */
+        Transitivity getTransitivity();
+
+        /**
+         * Returns the phase in which this module requirement is relevant.
+         *
+         * @return The phase in which this module requirement is relevant.
+         */
+        RequiredPhase getRequiredPhase();
+    }
+
     /**
      * An abstract base implementation of a {@link ModifierReviewable} class.
      */
-    abstract class AbstractBase implements ForTypeDefinition, ForFieldDescription, ForMethodDescription, ForParameterDescription {
+    abstract class AbstractBase implements ForTypeDefinition,
+            ForFieldDescription,
+            ForMethodDescription,
+            ForParameterDescription,
+            ForModuleDescription,
+            ForModuleRequirement {
 
         /**
          * {@inheritDoc}
@@ -448,6 +535,27 @@ public interface ModifierReviewable {
             return matchesMask(Opcodes.ACC_VARARGS);
         }
 
+        /**
+         * {@inheritDoc}
+         */
+        public boolean isOpen() {
+            return matchesMask(Opcodes.ACC_OPEN);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public boolean isStaticPhase() {
+            return matchesMask(Opcodes.ACC_STATIC_PHASE);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public boolean isTransitive() {
+            return matchesMask(Opcodes.ACC_TRANSITIVE);
+        }
+
         /**
          * {@inheritDoc}
          */
@@ -457,6 +565,15 @@ public interface ModifierReviewable {
                     : SyntheticState.PLAIN;
         }
 
+        /**
+         * {@inheritDoc}
+         */
+        public Mandate getMandate() {
+            return isMandated()
+                    ? Mandate.MANDATED
+                    : Mandate.PLAIN;
+        }
+
         /**
          * {@inheritDoc}
          */
@@ -608,6 +725,33 @@ public interface ModifierReviewable {
                     : ProvisioningState.PLAIN;
         }
 
+        /**
+         * {@inheritDoc}
+         */
+        public Openness getOpenness() {
+            return isOpen()
+                    ? Openness.OPEN
+                    : Openness.CLOSED;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public Transitivity getTransitivity() {
+            return isTransitive()
+                    ? Transitivity.TRANSITIVE
+                    : Transitivity.NONE;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public RequiredPhase getRequiredPhase() {
+            return isStaticPhase()
+                    ? RequiredPhase.STATIC
+                    : RequiredPhase.ALWAYS;
+        }
+
         /**
          * Checks if a mask is matched by this instance.
          *
