@@ -1,3 +1,18 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation;
 
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
@@ -8,10 +23,12 @@ import net.bytebuddy.description.type.TypeList;
 import net.bytebuddy.dynamic.scaffold.FieldLocator;
 import net.bytebuddy.dynamic.scaffold.InstrumentedType;
 import net.bytebuddy.implementation.bytecode.ByteCodeAppender;
+import net.bytebuddy.implementation.bytecode.Removal;
 import net.bytebuddy.implementation.bytecode.StackManipulation;
 import net.bytebuddy.implementation.bytecode.assign.Assigner;
 import net.bytebuddy.implementation.bytecode.collection.ArrayFactory;
 import net.bytebuddy.implementation.bytecode.constant.MethodConstant;
+import net.bytebuddy.implementation.bytecode.constant.NullConstant;
 import net.bytebuddy.implementation.bytecode.member.FieldAccess;
 import net.bytebuddy.implementation.bytecode.member.MethodInvocation;
 import net.bytebuddy.implementation.bytecode.member.MethodReturn;
@@ -24,7 +41,9 @@ import java.lang.reflect.InvocationHandler;
 import java.util.ArrayList;
 import java.util.List;
 
+import static net.bytebuddy.matcher.ElementMatchers.fieldType;
 import static net.bytebuddy.matcher.ElementMatchers.genericFieldType;
+import static net.bytebuddy.matcher.ElementMatchers.isAbstract;
 import static net.bytebuddy.matcher.ElementMatchers.named;
 
 /**
@@ -32,7 +51,7 @@ import static net.bytebuddy.matcher.ElementMatchers.named;
  * to also intercept method calls to non-interface methods.
  */
 @HashCodeAndEqualsPlugin.Enhance
-public abstract class InvocationHandlerAdapter implements Implementation {
+public abstract class InvocationHandlerAdapter implements Implementation.Composable {
 
     /**
      * A type description of the {@link InvocationHandler}.
@@ -50,26 +69,30 @@ public abstract class InvocationHandlerAdapter implements Implementation {
     private static final boolean CACHED = true;
 
     /**
-     * Indicates that a lookup of a method constant should not be looked up using an {@link java.security.AccessController}.
+     * Indicates that a lookup of a method constant should not be looked up using an {@code java.security.AccessController}.
      */
     private static final boolean UNPRIVILEGED = false;
 
     /**
-     * Indicates that a lookup of a method constant should be looked up using an {@link java.security.AccessController}.
+     * Indicates that a lookup of a method constant should be looked up using an {@code java.security.AccessController}.
      */
     private static final boolean PRIVILEGED = true;
 
+    /**
+     * Indicates that a handler is returning its return value.
+     */
+    private static final boolean RETURNING = true;
+
+    /**
+     * Indicates that a handler is dropping its return value.
+     */
+    private static final boolean DROPPING = false;
+
     /**
      * The name of the field for storing an invocation handler.
      */
     protected final String fieldName;
 
-    /**
-     * The assigner that is used for assigning the return invocation handler's return value to the
-     * intercepted method's return value.
-     */
-    protected final Assigner assigner;
-
     /**
      * Determines if the {@link java.lang.reflect.Method} instances that are handed to the intercepted methods are
      * cached in {@code static} fields.
@@ -77,23 +100,36 @@ public abstract class InvocationHandlerAdapter implements Implementation {
     protected final boolean cached;
 
     /**
-     * Determines if the {@link java.lang.reflect.Method} instances are retrieved by using an {@link java.security.AccessController}.
+     * Determines if the {@link java.lang.reflect.Method} instances are retrieved by using an {@code java.security.AccessController}.
      */
     protected final boolean privileged;
 
+    /**
+     * Determines if this implementation is returning the result value or is dropping it.
+     */
+    protected final boolean returning;
+
+    /**
+     * The assigner that is used for assigning the return invocation handler's return value to the
+     * intercepted method's return value.
+     */
+    protected final Assigner assigner;
+
     /**
      * Creates a new invocation handler for a given field.
      *
      * @param fieldName  The name of the field.
      * @param cached     Determines if the {@link java.lang.reflect.Method} instances that are handed to the
      *                   intercepted methods are cached in {@code static} fields.
-     * @param privileged Determines if the {@link java.lang.reflect.Method} instances are retrieved by using an {@link java.security.AccessController}.
+     * @param privileged Determines if the {@link java.lang.reflect.Method} instances are retrieved by using an {@code java.security.AccessController}.
+     * @param returning  Determines if this implementation is returning the result value or is dropping it.
      * @param assigner   The assigner to apply when defining this implementation.
      */
-    protected InvocationHandlerAdapter(String fieldName, boolean cached, boolean privileged, Assigner assigner) {
+    protected InvocationHandlerAdapter(String fieldName, boolean cached, boolean privileged, boolean returning, Assigner assigner) {
         this.fieldName = fieldName;
         this.cached = cached;
         this.privileged = privileged;
+        this.returning = returning;
         this.assigner = assigner;
     }
 
@@ -105,7 +141,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
      * @return An implementation that delegates all method interceptions to the given invocation handler.
      */
     public static InvocationHandlerAdapter of(InvocationHandler invocationHandler) {
-        return of(invocationHandler, ForInstance.PREFIX + "$" + RandomString.hashOf(invocationHandler.hashCode()));
+        return of(invocationHandler, ForInstance.PREFIX + "$" + RandomString.hashOf(invocationHandler));
     }
 
     /**
@@ -117,7 +153,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
      * @return An implementation that delegates all method interceptions to the given invocation handler.
      */
     public static InvocationHandlerAdapter of(InvocationHandler invocationHandler, String fieldName) {
-        return new ForInstance(fieldName, CACHED, UNPRIVILEGED, Assigner.DEFAULT, invocationHandler);
+        return new ForInstance(fieldName, CACHED, UNPRIVILEGED, RETURNING, Assigner.DEFAULT, invocationHandler);
     }
 
     /**
@@ -144,7 +180,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
      * @return An implementation that delegates all method interceptions to an instance field of the given name.
      */
     public static InvocationHandlerAdapter toField(String name, FieldLocator.Factory fieldLocatorFactory) {
-        return new ForField(name, CACHED, UNPRIVILEGED, Assigner.DEFAULT, fieldLocatorFactory);
+        return new ForField(name, CACHED, UNPRIVILEGED, RETURNING, Assigner.DEFAULT, fieldLocatorFactory);
     }
 
     /**
@@ -160,7 +196,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
         for (TypeDescription.Generic parameterType : parameterTypes) {
             instruction.add(new StackManipulation.Compound(
                     MethodVariableAccess.of(parameterType).loadFrom(currentIndex),
-                    assigner.assign(parameterType, TypeDescription.Generic.OBJECT, Assigner.Typing.STATIC)));
+                    assigner.assign(parameterType, TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Object.class), Assigner.Typing.STATIC)));
             currentIndex += parameterType.getStackSize().getSize();
         }
         return instruction;
@@ -184,7 +220,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
     public abstract Implementation withAssigner(Assigner assigner);
 
     /**
-     * Configures that the method constants supplied to the invocation handler adapter are resolved using an {@link java.security.AccessController}.
+     * Configures that the method constants supplied to the invocation handler adapter are resolved using an {@code java.security.AccessController}.
      *
      * @return This instrumentation with a privileged lookup configured.
      */
@@ -205,8 +241,8 @@ public abstract class InvocationHandlerAdapter implements Implementation {
                                           MethodDescription instrumentedMethod,
                                           StackManipulation preparingManipulation,
                                           FieldDescription fieldDescription) {
-        if (instrumentedMethod.isStatic()) {
-            throw new IllegalStateException("It is not possible to apply an invocation handler onto the static method " + instrumentedMethod);
+        if (instrumentedMethod.isStatic() || instrumentedMethod.isConstructor()) {
+            throw new IllegalStateException("It is not possible to apply an invocation handler onto the static method or constructor " + instrumentedMethod);
         }
         MethodConstant.CanCache methodConstant = privileged
                 ? MethodConstant.ofPrivileged(instrumentedMethod.asDefined())
@@ -216,10 +252,13 @@ public abstract class InvocationHandlerAdapter implements Implementation {
                 FieldAccess.forField(fieldDescription).read(),
                 MethodVariableAccess.loadThis(),
                 cached ? methodConstant.cached() : methodConstant,
-                ArrayFactory.forType(TypeDescription.Generic.OBJECT).withValues(argumentValuesOf(instrumentedMethod)),
-                MethodInvocation.invoke(INVOCATION_HANDLER_TYPE.getDeclaredMethods().getOnly()),
-                assigner.assign(TypeDescription.Generic.OBJECT, instrumentedMethod.getReturnType(), Assigner.Typing.DYNAMIC),
-                MethodReturn.of(instrumentedMethod.getReturnType())
+                instrumentedMethod.getParameters().isEmpty()
+                        ? NullConstant.INSTANCE
+                        : ArrayFactory.forType(TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Object.class)).withValues(argumentValuesOf(instrumentedMethod)),
+                MethodInvocation.invoke(INVOCATION_HANDLER_TYPE.getDeclaredMethods().filter(isAbstract()).getOnly()),
+                returning ? new StackManipulation.Compound(assigner.assign(TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Object.class),
+                        instrumentedMethod.getReturnType(),
+                        Assigner.Typing.DYNAMIC), MethodReturn.of(instrumentedMethod.getReturnType())) : Removal.SINGLE
         ).apply(methodVisitor, implementationContext);
         return new ByteCodeAppender.Size(stackSize.getMaximalSize(), instrumentedMethod.getStackSize());
     }
@@ -228,7 +267,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
      * Allows for the configuration of an {@link net.bytebuddy.implementation.bytecode.assign.Assigner}
      * of an {@link net.bytebuddy.implementation.InvocationHandlerAdapter}.
      */
-    public interface AssignerConfigurable extends Implementation {
+    public interface AssignerConfigurable extends Implementation.Composable {
 
         /**
          * Configures an assigner to use with this invocation handler adapter.
@@ -236,7 +275,7 @@ public abstract class InvocationHandlerAdapter implements Implementation {
          * @param assigner The assigner to apply when defining this implementation.
          * @return This instrumentation with the given {@code assigner} configured.
          */
-        Implementation withAssigner(Assigner assigner);
+        Implementation.Composable withAssigner(Assigner assigner);
     }
 
     /**
@@ -246,7 +285,8 @@ public abstract class InvocationHandlerAdapter implements Implementation {
     public interface WithoutPrivilegeConfiguration extends AssignerConfigurable {
 
         /**
-         * Configures that the method constants supplied to the invocation handler adapter are resolved using an {@link java.security.AccessController}.
+         * Configures that the method constants supplied to the invocation handler adapter are resolved
+         * using an {@code java.security.AccessController}.
          *
          * @return This instrumentation with a privileged lookup configured.
          */
@@ -278,32 +318,54 @@ public abstract class InvocationHandlerAdapter implements Implementation {
          * @param cached            Determines if the {@link java.lang.reflect.Method} instances that are handed to the
          *                          intercepted methods are cached in {@code static} fields.
          * @param privileged        Determines if the {@link java.lang.reflect.Method} instances are retrieved by
-         *                          using an {@link java.security.AccessController}.
+         *                          using an {@code java.security.AccessController}.
+         * @param returning         Determines if this implementation is returning the result value or is dropping it.
          * @param assigner          The assigner to apply when defining this implementation.
          * @param invocationHandler The invocation handler to which all method calls are delegated.
          */
-        protected ForInstance(String fieldName, boolean cached, boolean privileged, Assigner assigner, InvocationHandler invocationHandler) {
-            super(fieldName, cached, privileged, assigner);
+        protected ForInstance(String fieldName, boolean cached, boolean privileged, boolean returning, Assigner assigner, InvocationHandler invocationHandler) {
+            super(fieldName, cached, privileged, returning, assigner);
             this.invocationHandler = invocationHandler;
         }
 
         @Override
         public WithoutPrivilegeConfiguration withoutMethodCache() {
-            return new ForInstance(fieldName, UNCACHED, privileged, assigner, invocationHandler);
+            return new ForInstance(fieldName, UNCACHED, privileged, returning, assigner, invocationHandler);
         }
 
         @Override
-        public Implementation withAssigner(Assigner assigner) {
-            return new ForInstance(fieldName, cached, privileged, assigner, invocationHandler);
+        public Implementation.Composable withAssigner(Assigner assigner) {
+            return new ForInstance(fieldName, cached, privileged, returning, assigner, invocationHandler);
         }
 
         @Override
         public AssignerConfigurable withPrivilegedLookup() {
-            return new ForInstance(fieldName, cached, PRIVILEGED, assigner, invocationHandler);
+            return new ForInstance(fieldName, cached, PRIVILEGED, returning, assigner, invocationHandler);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        public Implementation andThen(Implementation implementation) {
+            return new Compound(new ForInstance(fieldName, cached, privileged, DROPPING, assigner, invocationHandler), implementation);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public Composable andThen(Composable implementation) {
+            return new Compound.Composable(new ForInstance(fieldName, cached, privileged, DROPPING, assigner, invocationHandler), implementation);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
+            if (!instrumentedType.getDeclaredFields().filter(named(fieldName).and(fieldType(INVOCATION_HANDLER_TYPE.asErasure()))).isEmpty()) {
+                throw new IllegalStateException("Field with name " + fieldName
+                        + " and type " + INVOCATION_HANDLER_TYPE.asErasure()
+                        + " already declared by " + instrumentedType);
+            }
             return instrumentedType
                     .withField(new FieldDescription.Token(fieldName,
                             Opcodes.ACC_PUBLIC | Opcodes.ACC_STATIC | Opcodes.ACC_VOLATILE | Opcodes.ACC_SYNTHETIC,
@@ -311,7 +373,9 @@ public abstract class InvocationHandlerAdapter implements Implementation {
                     .withInitializer(new LoadedTypeInitializer.ForStaticField(fieldName, invocationHandler));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             return new Appender(implementationTarget.getInstrumentedType());
         }
@@ -336,7 +400,9 @@ public abstract class InvocationHandlerAdapter implements Implementation {
                 this.instrumentedType = instrumentedType;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                 return ForInstance.this.apply(methodVisitor,
                         implementationContext,
@@ -366,36 +432,55 @@ public abstract class InvocationHandlerAdapter implements Implementation {
          * @param cached              Determines if the {@link java.lang.reflect.Method} instances that are handed to the
          *                            intercepted methods are cached in {@code static} fields.
          * @param privileged          Determines if the {@link java.lang.reflect.Method} instances are retrieved by using
-         *                            an {@link java.security.AccessController}.
+         *                            an {@code java.security.AccessController}.
+         * @param returning           Determines if this implementation is returning the result value or is dropping it.
          * @param assigner            The assigner to apply when defining this implementation.
          * @param fieldLocatorFactory The field locator factory to use.
          */
-        protected ForField(String fieldName, boolean cached, boolean privileged, Assigner assigner, FieldLocator.Factory fieldLocatorFactory) {
-            super(fieldName, cached, privileged, assigner);
+        protected ForField(String fieldName, boolean cached, boolean privileged, boolean returning, Assigner assigner, FieldLocator.Factory fieldLocatorFactory) {
+            super(fieldName, cached, privileged, returning, assigner);
             this.fieldLocatorFactory = fieldLocatorFactory;
         }
 
         @Override
         public WithoutPrivilegeConfiguration withoutMethodCache() {
-            return new ForField(fieldName, UNCACHED, privileged, assigner, fieldLocatorFactory);
+            return new ForField(fieldName, UNCACHED, privileged, returning, assigner, fieldLocatorFactory);
         }
 
         @Override
-        public Implementation withAssigner(Assigner assigner) {
-            return new ForField(fieldName, cached, privileged, assigner, fieldLocatorFactory);
+        public Implementation.Composable withAssigner(Assigner assigner) {
+            return new ForField(fieldName, cached, privileged, returning, assigner, fieldLocatorFactory);
         }
 
         @Override
         public AssignerConfigurable withPrivilegedLookup() {
-            return new ForField(fieldName, cached, PRIVILEGED, assigner, fieldLocatorFactory);
+            return new ForField(fieldName, cached, PRIVILEGED, returning, assigner, fieldLocatorFactory);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        public Implementation andThen(Implementation implementation) {
+            return new Compound(new ForField(fieldName, cached, privileged, DROPPING, assigner, fieldLocatorFactory), implementation);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public Composable andThen(Composable implementation) {
+            return new Compound.Composable(new ForField(fieldName, cached, privileged, DROPPING, assigner, fieldLocatorFactory), implementation);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             FieldLocator.Resolution resolution = fieldLocatorFactory.make(implementationTarget.getInstrumentedType()).locate(fieldName);
             if (!resolution.isResolved()) {
@@ -426,7 +511,9 @@ public abstract class InvocationHandlerAdapter implements Implementation {
                 this.fieldDescription = fieldDescription;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                 return ForField.this.apply(methodVisitor,
                         implementationContext,
