@@ -1,5 +1,21 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation;
 
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
 import net.bytebuddy.description.field.FieldDescription;
 import net.bytebuddy.description.method.MethodDescription;
@@ -16,13 +32,18 @@ import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.implementation.bytecode.member.MethodVariableAccess;
 import net.bytebuddy.matcher.ElementMatcher;
 import net.bytebuddy.matcher.ElementMatchers;
+import net.bytebuddy.utility.nullability.MaybeNull;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
 
 import java.util.ArrayList;
 import java.util.List;
 
-import static net.bytebuddy.matcher.ElementMatchers.*;
+import static net.bytebuddy.matcher.ElementMatchers.isConstructor;
+import static net.bytebuddy.matcher.ElementMatchers.isStatic;
+import static net.bytebuddy.matcher.ElementMatchers.isToString;
+import static net.bytebuddy.matcher.ElementMatchers.none;
+import static net.bytebuddy.matcher.ElementMatchers.not;
 
 /**
  * An implementation of {@link Object#toString()} that concatenates the {@link String} representation of all fields that are declared by a class.
@@ -186,12 +207,16 @@ public class ToStringMethod implements Implementation {
         return new ToStringMethod(prefixResolver, start, end, separator, definer, ignored);
     }
 
-    @Override
+    /**
+     * {@inheritDoc}
+     */
     public InstrumentedType prepare(InstrumentedType instrumentedType) {
         return instrumentedType;
     }
 
-    @Override
+    /**
+     * {@inheritDoc}
+     */
     public Appender appender(Target implementationTarget) {
         if (implementationTarget.getInstrumentedType().isInterface()) {
             throw new IllegalStateException("Cannot implement meaningful toString method for " + implementationTarget.getInstrumentedType());
@@ -268,7 +293,9 @@ public class ToStringMethod implements Implementation {
             this.fieldDescriptions = fieldDescriptions;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
             if (instrumentedMethod.isStatic()) {
                 throw new IllegalStateException("toString method must not be static: " + instrumentedMethod);
@@ -315,6 +342,7 @@ public class ToStringMethod implements Implementation {
          * @param instrumentedType The instrumented type.
          * @return The value to be prefixed.
          */
+        @MaybeNull
         String resolve(TypeDescription instrumentedType);
 
         /**
@@ -326,7 +354,7 @@ public class ToStringMethod implements Implementation {
              * A prefix resolver for the instrumented type's fully qualified class name.
              */
             FULLY_QUALIFIED_CLASS_NAME {
-                @Override
+                /** {@inheritDoc} */
                 public String resolve(TypeDescription instrumentedType) {
                     return instrumentedType.getName();
                 }
@@ -336,7 +364,8 @@ public class ToStringMethod implements Implementation {
              * A prefix resolver for the instrumented type's fully qualified class name.
              */
             CANONICAL_CLASS_NAME {
-                @Override
+                /** {@inheritDoc} */
+                @MaybeNull
                 public String resolve(TypeDescription instrumentedType) {
                     return instrumentedType.getCanonicalName();
                 }
@@ -346,7 +375,7 @@ public class ToStringMethod implements Implementation {
              * A prefix resolver for the instrumented type's simple class name.
              */
             SIMPLE_CLASS_NAME {
-                @Override
+                /** {@inheritDoc} */
                 public String resolve(TypeDescription instrumentedType) {
                     return instrumentedType.getSimpleName();
                 }
@@ -373,7 +402,9 @@ public class ToStringMethod implements Implementation {
                 this.prefix = prefix;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public String resolve(TypeDescription instrumentedType) {
                 return prefix;
             }
@@ -389,10 +420,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code boolean} value.
          */
         BOOLEAN {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Z)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -400,10 +431,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code char} value.
          */
         CHARACTER {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(C)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -411,10 +442,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for an {@code int} value.
          */
         INTEGER {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(I)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -422,7 +453,7 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code long} value.
          */
         LONG {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(J)Ljava/lang/StringBuilder;", false);
                 return new Size(-1, 0);
@@ -433,10 +464,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code float} value.
          */
         FLOAT {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(F)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -444,7 +475,7 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code double} value.
          */
         DOUBLE {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(D)Ljava/lang/StringBuilder;", false);
                 return new Size(-1, 0);
@@ -455,10 +486,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@link String} value.
          */
         STRING {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -467,10 +498,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@link CharSequence} value.
          */
         CHARACTER_SEQUENCE {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/CharSequence;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -478,10 +509,10 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a reference type.
          */
         OBJECT {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/Object;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -489,11 +520,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code boolean} array type.
          */
         BOOLEAN_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([Z)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -501,11 +532,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code byte} array type.
          */
         BYTE_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([B)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -513,11 +544,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code short} array type.
          */
         SHORT_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([S)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -525,11 +556,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code char} array type.
          */
         CHARACTER_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([C)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -537,11 +568,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for an {@code int} array type.
          */
         INTEGER_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([I)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -549,11 +580,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code long} array type.
          */
         LONG_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([J)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -561,11 +592,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code float} array type.
          */
         FLOAT_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([F)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -573,11 +604,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a {@code double} array type.
          */
         DOUBLE_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([D)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -585,11 +616,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a reference array type.
          */
         REFERENCE_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "toString", "([Ljava/lang/Object;)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         },
 
@@ -597,11 +628,11 @@ public class ToStringMethod implements Implementation {
          * A value consumer for a nested array type.
          */
         NESTED_ARRAY {
-            @Override
+            /** {@inheritDoc} */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC, "java/util/Arrays", "deepToString", "([Ljava/lang/Object;)Ljava/lang/String;", false);
                 methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL, "java/lang/StringBuilder", "append", "(Ljava/lang/String;)Ljava/lang/StringBuilder;", false);
-                return new Size(0, 0);
+                return Size.ZERO;
             }
         };
 
@@ -611,6 +642,7 @@ public class ToStringMethod implements Implementation {
          * @param typeDescription The type for which to resolve a value resolver.
          * @return An appropriate stack manipulation.
          */
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming component type for array type.")
         protected static StackManipulation of(TypeDescription typeDescription) {
             if (typeDescription.represents(boolean.class)) {
                 return BOOLEAN;
@@ -655,7 +687,9 @@ public class ToStringMethod implements Implementation {
             }
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public boolean isValid() {
             return true;
         }
