@@ -1,6 +1,20 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation.auxiliary;
 
-import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.ClassFileVersion;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
@@ -30,11 +44,16 @@ import net.bytebuddy.implementation.bytecode.member.FieldAccess;
 import net.bytebuddy.implementation.bytecode.member.MethodInvocation;
 import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.implementation.bytecode.member.MethodVariableAccess;
+import net.bytebuddy.utility.RandomString;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
 
 import java.io.Serializable;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.LinkedHashMap;
+import java.util.List;
+import java.util.Map;
 import java.util.concurrent.Callable;
 
 import static net.bytebuddy.matcher.ElementMatchers.isConstructor;
@@ -130,7 +149,16 @@ public class MethodCallProxy implements AuxiliaryType {
         return FIELD_NAME_PREFIX + index;
     }
 
-    @Override
+    /**
+     * {@inheritDoc}
+     */
+    public String getSuffix() {
+        return RandomString.hashOf(specialMethodInvocation.getMethodDescription().hashCode()) + (serializableProxy ? "S" : "0");
+    }
+
+    /**
+     * {@inheritDoc}
+     */
     public DynamicType make(String auxiliaryTypeName,
                             ClassFileVersion classFileVersion,
                             MethodAccessorFactory methodAccessorFactory) {
@@ -165,19 +193,18 @@ public class MethodCallProxy implements AuxiliaryType {
         /**
          * The precomputed method graph.
          */
-        private final MethodGraph.Linked methodGraph;
+        private final transient MethodGraph.Linked methodGraph;
 
         /**
          * Creates the precomputed method graph.
          */
-        @SuppressFBWarnings(value = "SE_BAD_FIELD_STORE", justification = "Precomputed method graph is not intended for serialization")
         PrecomputedMethodGraph() {
             LinkedHashMap<MethodDescription.SignatureToken, MethodGraph.Node> nodes = new LinkedHashMap<MethodDescription.SignatureToken, MethodGraph.Node>();
             MethodDescription callMethod = new MethodDescription.Latent(TypeDescription.ForLoadedType.of(Callable.class),
                     "call",
                     Opcodes.ACC_PUBLIC | Opcodes.ACC_ABSTRACT,
                     Collections.<TypeVariableToken>emptyList(),
-                    TypeDescription.Generic.OBJECT,
+                    TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Object.class),
                     Collections.<ParameterDescription.Token>emptyList(),
                     Collections.singletonList(TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Exception.class)),
                     Collections.<AnnotationDescription>emptyList(),
@@ -188,7 +215,7 @@ public class MethodCallProxy implements AuxiliaryType {
                     "run",
                     Opcodes.ACC_PUBLIC | Opcodes.ACC_ABSTRACT,
                     Collections.<TypeVariableToken>emptyList(),
-                    TypeDescription.Generic.VOID,
+                    TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(void.class),
                     Collections.<ParameterDescription.Token>emptyList(),
                     Collections.<TypeDescription.Generic>emptyList(),
                     Collections.<AnnotationDescription>emptyList(),
@@ -199,15 +226,35 @@ public class MethodCallProxy implements AuxiliaryType {
             this.methodGraph = new MethodGraph.Linked.Delegation(methodGraph, methodGraph, Collections.<TypeDescription, MethodGraph>emptyMap());
         }
 
-        @Override
-        public MethodGraph.Linked compile(TypeDescription typeDescription) {
-            return compile(typeDescription, typeDescription);
+        /**
+         * {@inheritDoc}
+         */
+        public MethodGraph.Linked compile(TypeDefinition typeDefinition) {
+            return methodGraph;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Deprecated
+        public MethodGraph.Linked compile(TypeDescription typeDescription) {
+            return methodGraph;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
         public MethodGraph.Linked compile(TypeDefinition typeDefinition, TypeDescription viewPoint) {
             return methodGraph;
         }
+
+        /**
+         * {@inheritDoc}
+         */
+        @Deprecated
+        public MethodGraph.Linked compile(TypeDescription typeDefinition, TypeDescription viewPoint) {
+            return methodGraph;
+        }
     }
 
     /**
@@ -229,15 +276,19 @@ public class MethodCallProxy implements AuxiliaryType {
          * Creates the constructor call singleton.
          */
         ConstructorCall() {
-            objectTypeDefaultConstructor = TypeDescription.OBJECT.getDeclaredMethods().filter(isConstructor()).getOnly();
+            objectTypeDefaultConstructor = TypeDescription.ForLoadedType.of(Object.class).getDeclaredMethods().filter(isConstructor()).getOnly();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             return new Appender(implementationTarget.getInstrumentedType());
         }
@@ -262,7 +313,9 @@ public class MethodCallProxy implements AuxiliaryType {
                 this.instrumentedType = instrumentedType;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                 FieldList<?> fieldList = instrumentedType.getDeclaredFields();
                 StackManipulation[] fieldLoading = new StackManipulation[fieldList.size()];
@@ -313,12 +366,16 @@ public class MethodCallProxy implements AuxiliaryType {
             this.assigner = assigner;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             return new Appender(implementationTarget.getInstrumentedType());
         }
@@ -343,7 +400,9 @@ public class MethodCallProxy implements AuxiliaryType {
                 this.instrumentedType = instrumentedType;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor,
                               Context implementationContext,
                               MethodDescription instrumentedMethod) {
@@ -370,7 +429,7 @@ public class MethodCallProxy implements AuxiliaryType {
      * as the target method itself.
      */
     @HashCodeAndEqualsPlugin.Enhance
-    public static class AssignableSignatureCall implements StackManipulation {
+    public static class AssignableSignatureCall extends StackManipulation.AbstractBase {
 
         /**
          * The special method invocation to be proxied by this stack manipulation.
@@ -397,12 +456,9 @@ public class MethodCallProxy implements AuxiliaryType {
             this.serializable = serializable;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             TypeDescription auxiliaryType = implementationContext
                     .register(new MethodCallProxy(specialMethodInvocation, serializable));
