@@ -1,6 +1,20 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation.bind.annotation;
 
-import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.ClassFileVersion;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
@@ -31,14 +45,22 @@ import net.bytebuddy.implementation.bytecode.member.MethodInvocation;
 import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.implementation.bytecode.member.MethodVariableAccess;
 import net.bytebuddy.matcher.ElementMatchers;
+import net.bytebuddy.utility.RandomString;
 import org.objectweb.asm.MethodVisitor;
 
 import java.io.Serializable;
-import java.lang.annotation.*;
+import java.lang.annotation.Documented;
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
 import java.util.LinkedHashMap;
 import java.util.Map;
 
-import static net.bytebuddy.matcher.ElementMatchers.*;
+import static net.bytebuddy.matcher.ElementMatchers.isAbstract;
+import static net.bytebuddy.matcher.ElementMatchers.isConstructor;
+import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
+import static net.bytebuddy.matcher.ElementMatchers.named;
 
 /**
  * <p>
@@ -78,6 +100,14 @@ public @interface Pipe {
     @HashCodeAndEqualsPlugin.Enhance
     class Binder implements TargetMethodAnnotationDrivenBinder.ParameterBinder<Pipe> {
 
+        /**
+         * A description of the {@link Pipe#serializableProxy()} method.
+         */
+        private static final MethodDescription.InDefinedShape SERIALIZABLE_PROXY = TypeDescription.ForLoadedType.of(Pipe.class)
+                .getDeclaredMethods()
+                .filter(named("serializableProxy"))
+                .getOnly();
+
         /**
          * The method which implements the behavior of forwarding a method invocation. This method needs to define
          * a single non-static method with an {@link java.lang.Object} to {@link java.lang.Object} mapping.
@@ -153,12 +183,16 @@ public @interface Pipe {
             return methodDescription;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Class<Pipe> getHandledType() {
             return Pipe.class;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public MethodDelegationBinder.ParameterBinding<?> bind(AnnotationDescription.Loadable<Pipe> annotation,
                                                                MethodDescription source,
                                                                ParameterDescription target,
@@ -170,10 +204,10 @@ public @interface Pipe {
             } else if (source.isStatic()) {
                 return MethodDelegationBinder.ParameterBinding.Illegal.INSTANCE;
             }
-            return new MethodDelegationBinder.ParameterBinding.Anonymous(new Redirection(forwardingMethod.getDeclaringType().asErasure(),
+            return new MethodDelegationBinder.ParameterBinding.Anonymous(new RedirectionProxy(forwardingMethod.getDeclaringType().asErasure(),
                     source,
                     assigner,
-                    annotation.loadSilent().serializableProxy()));
+                    annotation.getValue(SERIALIZABLE_PROXY).resolve(Boolean.class)));
         }
 
         /**
@@ -181,7 +215,7 @@ public @interface Pipe {
          * {@link net.bytebuddy.implementation.bind.annotation.Pipe} annotation.
          */
         @HashCodeAndEqualsPlugin.Enhance
-        protected static class Redirection implements AuxiliaryType, StackManipulation {
+        protected static class RedirectionProxy extends StackManipulation.AbstractBase implements AuxiliaryType {
 
             /**
              * The prefix for naming fields to store method arguments.
@@ -216,10 +250,10 @@ public @interface Pipe {
              * @param assigner          The assigner to use.
              * @param serializableProxy Determines if the generated proxy should be {@link java.io.Serializable}.
              */
-            protected Redirection(TypeDescription forwardingType,
-                                  MethodDescription sourceMethod,
-                                  Assigner assigner,
-                                  boolean serializableProxy) {
+            protected RedirectionProxy(TypeDescription forwardingType,
+                                       MethodDescription sourceMethod,
+                                       Assigner assigner,
+                                       boolean serializableProxy) {
                 this.forwardingType = forwardingType;
                 this.sourceMethod = sourceMethod;
                 this.assigner = assigner;
@@ -253,7 +287,18 @@ public @interface Pipe {
                 return FIELD_NAME_PREFIX + index;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
+            public String getSuffix() {
+                return RandomString.hashOf(forwardingType.hashCode())
+                        + RandomString.hashOf(sourceMethod.hashCode())
+                        + (serializableProxy ? "S" : "0");
+            }
+
+            /**
+             * {@inheritDoc}
+             */
             public DynamicType make(String auxiliaryTypeName,
                                     ClassFileVersion classFileVersion,
                                     MethodAccessorFactory methodAccessorFactory) {
@@ -274,12 +319,9 @@ public @interface Pipe {
                 return builder.make();
             }
 
-            @Override
-            public boolean isValid() {
-                return true;
-            }
-
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
                 TypeDescription forwardingType = implementationContext.register(this);
                 return new Compound(
@@ -291,11 +333,8 @@ public @interface Pipe {
             }
 
             /**
-             * The implementation to implement a
-             * {@link net.bytebuddy.implementation.bind.annotation.Pipe.Binder.Redirection}'s
-             * constructor.
+             * The implementation to implement a {@link RedirectionProxy}'s constructor.
              */
-            @SuppressFBWarnings(value = "SE_BAD_FIELD", justification = "Fields of enumerations are never serialized")
             protected enum ConstructorCall implements Implementation {
 
                 /**
@@ -306,28 +345,31 @@ public @interface Pipe {
                 /**
                  * A reference of the {@link Object} type default constructor.
                  */
-                private final MethodDescription.InDefinedShape objectTypeDefaultConstructor;
+                private final transient MethodDescription.InDefinedShape objectTypeDefaultConstructor;
 
                 /**
                  * Creates the constructor call singleton.
                  */
                 ConstructorCall() {
-                    objectTypeDefaultConstructor = TypeDescription.OBJECT.getDeclaredMethods().filter(isConstructor()).getOnly();
+                    objectTypeDefaultConstructor = TypeDescription.ForLoadedType.of(Object.class).getDeclaredMethods().filter(isConstructor()).getOnly();
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public InstrumentedType prepare(InstrumentedType instrumentedType) {
                     return instrumentedType;
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public ByteCodeAppender appender(Target implementationTarget) {
                     return new Appender(implementationTarget.getInstrumentedType());
                 }
 
                 /**
-                 * The appender for implementing the
-                 * {@link net.bytebuddy.implementation.bind.annotation.Pipe.Binder.Redirection.ConstructorCall}.
+                 * The appender for implementing the {@link RedirectionProxy.ConstructorCall}.
                  */
                 @HashCodeAndEqualsPlugin.Enhance
                 private static class Appender implements ByteCodeAppender {
@@ -346,7 +388,9 @@ public @interface Pipe {
                         this.instrumentedType = instrumentedType;
                     }
 
-                    @Override
+                    /**
+                     * {@inheritDoc}
+                     */
                     public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                         FieldList<?> fieldList = instrumentedType.getDeclaredFields();
                         StackManipulation[] fieldLoading = new StackManipulation[fieldList.size()];
@@ -371,9 +415,7 @@ public @interface Pipe {
             }
 
             /**
-             * The implementation to implement a
-             * {@link net.bytebuddy.implementation.bind.annotation.Pipe.Binder.Redirection}'s
-             * forwarding method.
+             * The implementation to implement a {@link RedirectionProxy}'s forwarding method.
              */
             @HashCodeAndEqualsPlugin.Enhance
             protected static class MethodCall implements Implementation {
@@ -399,12 +441,16 @@ public @interface Pipe {
                     this.assigner = assigner;
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public InstrumentedType prepare(InstrumentedType instrumentedType) {
                     return instrumentedType;
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public ByteCodeAppender appender(Target implementationTarget) {
                     if (!redirectedMethod.isAccessibleTo(implementationTarget.getInstrumentedType())) {
                         throw new IllegalStateException("Cannot invoke " + redirectedMethod + " from outside of class via @Pipe proxy");
@@ -413,8 +459,7 @@ public @interface Pipe {
                 }
 
                 /**
-                 * The appender for implementing the
-                 * {@link net.bytebuddy.implementation.bind.annotation.Pipe.Binder.Redirection.MethodCall}.
+                 * The appender for implementing the {@link RedirectionProxy.MethodCall}.
                  */
                 @HashCodeAndEqualsPlugin.Enhance(includeSyntheticFields = true)
                 private class Appender implements ByteCodeAppender {
@@ -433,20 +478,21 @@ public @interface Pipe {
                         this.instrumentedType = instrumentedType;
                     }
 
-                    @Override
+                    /**
+                     * {@inheritDoc}
+                     */
                     public Size apply(MethodVisitor methodVisitor,
                                       Context implementationContext,
                                       MethodDescription instrumentedMethod) {
-                        StackManipulation thisReference = MethodVariableAccess.of(instrumentedType).loadFrom(0);
                         FieldList<?> fieldList = instrumentedType.getDeclaredFields();
                         StackManipulation[] fieldLoading = new StackManipulation[fieldList.size()];
                         int index = 0;
                         for (FieldDescription fieldDescription : fieldList) {
-                            fieldLoading[index++] = new StackManipulation.Compound(thisReference, FieldAccess.forField(fieldDescription).read());
+                            fieldLoading[index++] = new StackManipulation.Compound(MethodVariableAccess.loadThis(), FieldAccess.forField(fieldDescription).read());
                         }
                         StackManipulation.Size stackSize = new StackManipulation.Compound(
                                 MethodVariableAccess.REFERENCE.loadFrom(1),
-                                assigner.assign(TypeDescription.Generic.OBJECT, redirectedMethod.getDeclaringType().asGenericType(), Assigner.Typing.DYNAMIC),
+                                assigner.assign(TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(Object.class), redirectedMethod.getDeclaringType().asGenericType(), Assigner.Typing.DYNAMIC),
                                 new StackManipulation.Compound(fieldLoading),
                                 MethodInvocation.invoke(redirectedMethod),
                                 assigner.assign(redirectedMethod.getReturnType(), instrumentedMethod.getReturnType(), Assigner.Typing.DYNAMIC),
