@@ -13,9 +13,20 @@ import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
 import net.bytebuddy.dynamic.loading.ClassReloadingStrategy;
 import net.bytebuddy.dynamic.scaffold.InstrumentedType;
 import net.bytebuddy.dynamic.scaffold.subclass.ConstructorStrategy;
-import net.bytebuddy.implementation.*;
+import net.bytebuddy.implementation.DefaultMethodCall;
+import net.bytebuddy.implementation.FieldAccessor;
+import net.bytebuddy.implementation.FixedValue;
+import net.bytebuddy.implementation.Implementation;
+import net.bytebuddy.implementation.MethodCall;
+import net.bytebuddy.implementation.MethodDelegation;
+import net.bytebuddy.implementation.SuperMethodCall;
 import net.bytebuddy.implementation.bind.MethodDelegationBinder;
-import net.bytebuddy.implementation.bind.annotation.*;
+import net.bytebuddy.implementation.bind.annotation.Argument;
+import net.bytebuddy.implementation.bind.annotation.Pipe;
+import net.bytebuddy.implementation.bind.annotation.RuntimeType;
+import net.bytebuddy.implementation.bind.annotation.Super;
+import net.bytebuddy.implementation.bind.annotation.SuperCall;
+import net.bytebuddy.implementation.bind.annotation.TargetMethodAnnotationDrivenBinder;
 import net.bytebuddy.implementation.bytecode.ByteCodeAppender;
 import net.bytebuddy.implementation.bytecode.StackManipulation;
 import net.bytebuddy.implementation.bytecode.assign.Assigner;
@@ -40,24 +51,28 @@ import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.lang.instrument.Instrumentation;
 import java.lang.reflect.Method;
-import java.net.URL;
-import java.net.URLClassLoader;
+import java.security.ProtectionDomain;
 import java.util.Arrays;
 import java.util.List;
 import java.util.concurrent.Callable;
 
-import static net.bytebuddy.matcher.ElementMatchers.*;
+import static net.bytebuddy.matcher.ElementMatchers.isAnnotatedWith;
+import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
+import static net.bytebuddy.matcher.ElementMatchers.named;
 import static net.bytebuddy.matcher.ElementMatchers.not;
+import static net.bytebuddy.matcher.ElementMatchers.returns;
+import static net.bytebuddy.matcher.ElementMatchers.takesArguments;
 import static org.hamcrest.CoreMatchers.is;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.startsWith;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.mockito.Mockito.mock;
 
 public class ByteBuddyTutorialExamplesTest {
 
-    private static final String DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.SingleDefaultMethodInterface";
+    private static final String DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.v8.SingleDefaultMethodInterface";
 
-    private static final String CONFLICTING_DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.SingleDefaultMethodConflictingInterface";
+    private static final String CONFLICTING_DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.v8.SingleDefaultMethodConflictingInterface";
 
     @Rule
     public MethodRule javaVersionRule = new JavaVersionRule();
@@ -154,12 +169,11 @@ public class ByteBuddyTutorialExamplesTest {
 
     @Test
     public void testTutorialGettingStartedTypePool() throws Exception {
-        TypePool typePool = TypePool.Default.ofClassPath();
-        ClassLoader classLoader = new URLClassLoader(new URL[0], null); // Assure repeatability.
-        Class<?> type = new ByteBuddy().redefine(typePool.describe(UnloadedBar.class.getName()).resolve(), ClassFileLocator.ForClassLoader.ofClassPath())
+        TypePool typePool = TypePool.Default.ofSystemLoader();
+        Class<?> type = new ByteBuddy().redefine(typePool.describe(UnloadedBar.class.getName()).resolve(), ClassFileLocator.ForClassLoader.ofSystemLoader())
                 .defineField("qux", String.class)
                 .make()
-                .load(classLoader, ClassLoadingStrategy.Default.WRAPPER)
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded();
         assertThat(type.getDeclaredField("qux"), notNullValue(java.lang.reflect.Field.class));
     }
@@ -167,8 +181,11 @@ public class ByteBuddyTutorialExamplesTest {
     @Test
     public void testTutorialGettingStartedJavaAgent() throws Exception {
         new AgentBuilder.Default().type(isAnnotatedWith(Rebase.class)).transform(new AgentBuilder.Transformer() {
-            @Override
-            public DynamicType.Builder<?> transform(DynamicType.Builder<?> builder, TypeDescription typeDescription, ClassLoader classLoader, JavaModule module) {
+            public DynamicType.Builder<?> transform(DynamicType.Builder<?> builder,
+                                                    TypeDescription typeDescription,
+                                                    ClassLoader classLoader,
+                                                    JavaModule module,
+                                                    ProtectionDomain protectionDomain) {
                 return builder.method(named("toString")).intercept(FixedValue.value("transformed"));
             }
         }).installOn(mock(Instrumentation.class));
@@ -189,7 +206,7 @@ public class ByteBuddyTutorialExamplesTest {
 
     @Test
     public void testFieldsAndMethodsDetailedMatcher() throws Exception {
-        assertThat(TypeDescription.OBJECT
+        assertThat(TypeDescription.ForLoadedType.of(Object.class)
                 .getDeclaredMethods()
                 .filter(named("toString").and(returns(String.class)).and(takesArguments(0))).size(), is(1));
     }
@@ -439,12 +456,10 @@ public class ByteBuddyTutorialExamplesTest {
 
         INSTANCE;
 
-        @Override
         public boolean isValid() {
             return true;
         }
 
-        @Override
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             methodVisitor.visitInsn(Opcodes.IADD);
             return new Size(-1, 0);
@@ -455,7 +470,6 @@ public class ByteBuddyTutorialExamplesTest {
 
         INSTANCE;
 
-        @Override
         public Size apply(MethodVisitor methodVisitor,
                           Implementation.Context implementationContext,
                           MethodDescription instrumentedMethod) {
@@ -475,12 +489,10 @@ public class ByteBuddyTutorialExamplesTest {
     public enum SumImplementation implements Implementation {
         INSTANCE;
 
-        @Override
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType;
         }
 
-        @Override
         public ByteCodeAppender appender(Target implementationTarget) {
             return SumMethod.INSTANCE;
         }
@@ -490,10 +502,9 @@ public class ByteBuddyTutorialExamplesTest {
 
         INSTANCE;
 
-        @Override
         public StackManipulation assign(TypeDescription.Generic source, TypeDescription.Generic target, Typing typing) {
             if (!source.isPrimitive() && target.represents(String.class)) {
-                MethodDescription toStringMethod = TypeDescription.OBJECT.getDeclaredMethods()
+                MethodDescription toStringMethod = TypeDescription.ForLoadedType.of(Object.class).getDeclaredMethods()
                         .filter(named("toString"))
                         .getOnly();
                 return MethodInvocation.invoke(toStringMethod).virtual(source.asErasure());
@@ -507,12 +518,10 @@ public class ByteBuddyTutorialExamplesTest {
 
         INSTANCE;
 
-        @Override
         public Class<StringValue> getHandledType() {
             return StringValue.class;
         }
 
-        @Override
         public MethodDelegationBinder.ParameterBinding<?> bind(AnnotationDescription.Loadable<StringValue> annotation,
                                                                MethodDescription source,
                                                                ParameterDescription target,
@@ -522,7 +531,7 @@ public class ByteBuddyTutorialExamplesTest {
             if (!target.getType().asErasure().represents(String.class)) {
                 throw new IllegalStateException(target + " makes wrong use of StringValue");
             }
-            StackManipulation constant = new TextConstant(annotation.loadSilent().value());
+            StackManipulation constant = new TextConstant(annotation.load().value());
             return new MethodDelegationBinder.ParameterBinding.Anonymous(constant);
         }
     }
@@ -749,7 +758,6 @@ public class ByteBuddyTutorialExamplesTest {
     @SuppressWarnings("unused")
     public static class HelloWorldInterceptor implements Interceptor2 {
 
-        @Override
         public String doSomethingElse() {
             return "Hello World!";
         }
@@ -757,7 +765,6 @@ public class ByteBuddyTutorialExamplesTest {
 
     private static class RuntimeDefinitionImpl implements RuntimeDefinition {
 
-        @Override
         public Class<? extends Annotation> annotationType() {
             return RuntimeDefinition.class;
         }
@@ -797,10 +804,9 @@ public class ByteBuddyTutorialExamplesTest {
         }
     }
 
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     class LoggingMemoryDatabase extends MemoryDatabase {
 
-        @Override
         public List<String> load(String info) {
             try {
                 return LoggerInterceptor.log(new LoadMethodSuperCall(info));
@@ -817,7 +823,6 @@ public class ByteBuddyTutorialExamplesTest {
                 this.info = info;
             }
 
-            @Override
             public Object call() throws Exception {
                 return LoggingMemoryDatabase.super.load(info);
             }
