@@ -1,25 +1,34 @@
 package net.bytebuddy.agent.builder;
 
+import net.bytebuddy.description.type.PackageDescription;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.DynamicType;
 import net.bytebuddy.matcher.ElementMatchers;
-import net.bytebuddy.test.utility.MockitoRule;
+import net.bytebuddy.test.utility.JavaVersionRule;
 import net.bytebuddy.utility.JavaModule;
+import net.bytebuddy.utility.JavaType;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.TestRule;
+import org.junit.rules.MethodRule;
 import org.mockito.Mock;
+import org.mockito.junit.MockitoJUnit;
 
 import java.io.PrintStream;
 import java.lang.instrument.Instrumentation;
 import java.util.Collections;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import static net.bytebuddy.matcher.ElementMatchers.none;
 import static net.bytebuddy.test.utility.FieldByFieldComparison.hasPrototype;
-import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class AgentBuilderListenerTest {
 
@@ -28,7 +37,10 @@ public class AgentBuilderListenerTest {
     private static final boolean LOADED = true;
 
     @Rule
-    public TestRule mockitoRule = new MockitoRule(this);
+    public MethodRule mockitoRule = MockitoJUnit.rule().silent();
+
+    @Rule
+    public MethodRule javaVersionRule = new JavaVersionRule();
 
     @Mock
     private AgentBuilder.Listener first, second;
@@ -57,9 +69,9 @@ public class AgentBuilderListenerTest {
     public void testNoOp() throws Exception {
         AgentBuilder.Listener.NoOp.INSTANCE.onDiscovery(FOO, classLoader, module, LOADED);
         AgentBuilder.Listener.NoOp.INSTANCE.onTransformation(typeDescription, classLoader, module, LOADED, dynamicType);
-        verifyZeroInteractions(dynamicType);
+        verifyNoMoreInteractions(dynamicType);
         AgentBuilder.Listener.NoOp.INSTANCE.onError(FOO, classLoader, module, LOADED, throwable);
-        verifyZeroInteractions(throwable);
+        verifyNoMoreInteractions(throwable);
         AgentBuilder.Listener.NoOp.INSTANCE.onIgnored(typeDescription, classLoader, module, LOADED);
         AgentBuilder.Listener.NoOp.INSTANCE.onComplete(FOO, classLoader, module, LOADED);
     }
@@ -69,9 +81,9 @@ public class AgentBuilderListenerTest {
         AgentBuilder.Listener listener = new PseudoAdapter();
         listener.onDiscovery(FOO, classLoader, module, LOADED);
         listener.onTransformation(typeDescription, classLoader, module, LOADED, dynamicType);
-        verifyZeroInteractions(dynamicType);
+        verifyNoMoreInteractions(dynamicType);
         listener.onError(FOO, classLoader, module, LOADED, throwable);
-        verifyZeroInteractions(throwable);
+        verifyNoMoreInteractions(throwable);
         listener.onIgnored(typeDescription, classLoader, module, LOADED);
         listener.onComplete(FOO, classLoader, module, LOADED);
     }
@@ -126,7 +138,7 @@ public class AgentBuilderListenerTest {
         PrintStream printStream = mock(PrintStream.class);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.StreamWriting(printStream);
         listener.onDiscovery(FOO, classLoader, module, LOADED);
-        verify(printStream).printf("[Byte Buddy] DISCOVERY %s [%s, %s, loaded=%b]%n", FOO, classLoader, module, LOADED);
+        verify(printStream).printf("[Byte Buddy] DISCOVERY %s [%s, %s, %s, loaded=%b]%n", FOO, classLoader, module, Thread.currentThread(), LOADED);
         verifyNoMoreInteractions(printStream);
     }
 
@@ -135,7 +147,7 @@ public class AgentBuilderListenerTest {
         PrintStream printStream = mock(PrintStream.class);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.StreamWriting(printStream);
         listener.onTransformation(typeDescription, classLoader, module, LOADED, dynamicType);
-        verify(printStream).printf("[Byte Buddy] TRANSFORM %s [%s, %s, loaded=%b]%n", FOO, classLoader, module, LOADED);
+        verify(printStream).printf("[Byte Buddy] TRANSFORM %s [%s, %s, %s, loaded=%b]%n", FOO, classLoader, module, Thread.currentThread(), LOADED);
         verifyNoMoreInteractions(printStream);
     }
 
@@ -144,7 +156,7 @@ public class AgentBuilderListenerTest {
         PrintStream printStream = mock(PrintStream.class);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.StreamWriting(printStream);
         listener.onError(FOO, classLoader, module, LOADED, throwable);
-        verify(printStream).printf("[Byte Buddy] ERROR %s [%s, %s, loaded=%b]%n", FOO, classLoader, module, LOADED);
+        verify(printStream).printf("[Byte Buddy] ERROR %s [%s, %s, %s, loaded=%b]%n", FOO, classLoader, module, Thread.currentThread(), LOADED);
         verifyNoMoreInteractions(printStream);
         verify(throwable).printStackTrace(printStream);
         verifyNoMoreInteractions(throwable);
@@ -155,7 +167,7 @@ public class AgentBuilderListenerTest {
         PrintStream printStream = mock(PrintStream.class);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.StreamWriting(printStream);
         listener.onComplete(FOO, classLoader, module, LOADED);
-        verify(printStream).printf("[Byte Buddy] COMPLETE %s [%s, %s, loaded=%b]%n", FOO, classLoader, module, LOADED);
+        verify(printStream).printf("[Byte Buddy] COMPLETE %s [%s, %s, %s, loaded=%b]%n", FOO, classLoader, module, Thread.currentThread(), LOADED);
         verifyNoMoreInteractions(printStream);
     }
 
@@ -164,7 +176,7 @@ public class AgentBuilderListenerTest {
         PrintStream printStream = mock(PrintStream.class);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.StreamWriting(printStream);
         listener.onIgnored(typeDescription, classLoader, module, LOADED);
-        verify(printStream).printf("[Byte Buddy] IGNORE %s [%s, %s, loaded=%b]%n", FOO, classLoader, module, LOADED);
+        verify(printStream).printf("[Byte Buddy] IGNORE %s [%s, %s, %s, loaded=%b]%n", FOO, classLoader, module, Thread.currentThread(), LOADED);
         verifyNoMoreInteractions(printStream);
     }
 
@@ -228,7 +240,7 @@ public class AgentBuilderListenerTest {
         listener.onError(FOO, classLoader, module, LOADED, throwable);
         listener.onIgnored(typeDescription, classLoader, module, LOADED);
         listener.onComplete(FOO, classLoader, module, LOADED);
-        verifyZeroInteractions(delegate);
+        verifyNoMoreInteractions(delegate);
     }
 
     @Test
@@ -263,7 +275,7 @@ public class AgentBuilderListenerTest {
         listener.onTransformation(mock(TypeDescription.class), mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
         verify(source).isNamed();
         verifyNoMoreInteractions(source);
-        verifyZeroInteractions(target);
+        verifyNoMoreInteractions(target);
     }
 
     @Test
@@ -277,22 +289,38 @@ public class AgentBuilderListenerTest {
         verify(source).isNamed();
         verify(source).canRead(target);
         verifyNoMoreInteractions(source);
-        verifyZeroInteractions(target);
+        verifyNoMoreInteractions(target);
     }
 
     @Test
+    @JavaVersionRule.Enforce(9)
     public void testReadEdgeAddingListenerNamedCannotRead() throws Exception {
         Instrumentation instrumentation = mock(Instrumentation.class);
         JavaModule source = mock(JavaModule.class), target = mock(JavaModule.class);
         when(source.isNamed()).thenReturn(true);
         when(source.canRead(target)).thenReturn(false);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.ModuleReadEdgeCompleting(instrumentation, false, Collections.singleton(target));
+        when(Instrumentation.class.getMethod("isModifiableModule", JavaType.MODULE.load()).invoke(instrumentation, (Object) null)).thenReturn(true);
         listener.onTransformation(mock(TypeDescription.class), mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
         verify(source).isNamed();
         verify(source).canRead(target);
-        verify(source).addReads(instrumentation, target);
+        Instrumentation.class.getMethod("redefineModule",
+                JavaType.MODULE.load(),
+                Set.class,
+                Map.class,
+                Map.class,
+                Set.class,
+                Map.class).invoke(verify(instrumentation),
+                null,
+                Collections.singleton(null),
+                Collections.<String, Set<JavaModule>>emptyMap(),
+                Collections.<String, Set<JavaModule>>emptyMap(),
+                Collections.<Class<?>>emptySet(),
+                Collections.<Class<?>, List<Class<?>>>emptyMap());
+        verify(source, times(2)).unwrap();
         verifyNoMoreInteractions(source);
-        verifyZeroInteractions(target);
+        verify(target).unwrap();
+        verifyNoMoreInteractions(target);
     }
 
     @Test
@@ -310,40 +338,60 @@ public class AgentBuilderListenerTest {
         listener.onTransformation(mock(TypeDescription.class), mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
         verify(source).isNamed();
         verifyNoMoreInteractions(source);
-        verifyZeroInteractions(target);
+        verifyNoMoreInteractions(target);
     }
 
     @Test
     public void testReadEdgeAddingListenerDuplexCanRead() throws Exception {
         Instrumentation instrumentation = mock(Instrumentation.class);
         JavaModule source = mock(JavaModule.class), target = mock(JavaModule.class);
+        TypeDescription typeDescription = mock(TypeDescription.class);
+        PackageDescription packageDescription = mock(PackageDescription.class);
+        when(typeDescription.getPackage()).thenReturn(packageDescription);
         when(source.isNamed()).thenReturn(true);
         when(source.canRead(target)).thenReturn(true);
+        when(source.isOpened(packageDescription, target)).thenReturn(true);
         when(target.canRead(source)).thenReturn(true);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.ModuleReadEdgeCompleting(instrumentation, true, Collections.singleton(target));
-        listener.onTransformation(mock(TypeDescription.class), mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
+        listener.onTransformation(typeDescription, mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
         verify(source).isNamed();
         verify(source).canRead(target);
+        verify(source).isOpened(packageDescription, target);
         verifyNoMoreInteractions(source);
         verify(target).canRead(source);
         verifyNoMoreInteractions(target);
     }
 
     @Test
+    @JavaVersionRule.Enforce(9)
     public void testReadEdgeAddingListenerNamedDuplexCannotRead() throws Exception {
         Instrumentation instrumentation = mock(Instrumentation.class);
         JavaModule source = mock(JavaModule.class), target = mock(JavaModule.class);
         when(source.isNamed()).thenReturn(true);
         when(source.canRead(target)).thenReturn(false);
         when(target.canRead(source)).thenReturn(false);
+        when(Instrumentation.class.getMethod("isModifiableModule", JavaType.MODULE.load()).invoke(instrumentation, (Object) null)).thenReturn(true);
         AgentBuilder.Listener listener = new AgentBuilder.Listener.ModuleReadEdgeCompleting(instrumentation, true, Collections.singleton(target));
         listener.onTransformation(mock(TypeDescription.class), mock(ClassLoader.class), source, LOADED, mock(DynamicType.class));
+        Instrumentation.class.getMethod("redefineModule",
+                JavaType.MODULE.load(),
+                Set.class,
+                Map.class,
+                Map.class,
+                Set.class,
+                Map.class).invoke(verify(instrumentation, times(2)),
+                null,
+                Collections.singleton(null),
+                Collections.<String, Set<JavaModule>>emptyMap(),
+                Collections.<String, Set<JavaModule>>emptyMap(),
+                Collections.<Class<?>>emptySet(),
+                Collections.<Class<?>, List<Class<?>>>emptyMap());
         verify(source).isNamed();
         verify(source).canRead(target);
-        verify(source).addReads(instrumentation, target);
+        verify(source, times(3)).unwrap();
         verifyNoMoreInteractions(source);
         verify(target).canRead(source);
-        verify(target).addReads(instrumentation, source);
+        verify(target, times(3)).unwrap();
         verifyNoMoreInteractions(target);
     }
 
