@@ -1,25 +1,21 @@
 package net.bytebuddy.dynamic;
 
 import net.bytebuddy.description.type.TypeDescription;
-import net.bytebuddy.test.utility.MockitoRule;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.TestRule;
+import org.junit.rules.MethodRule;
 import org.mockito.Mock;
-
-import java.util.Collections;
-import java.util.List;
+import org.mockito.junit.MockitoJUnit;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
 public class TransformerCompoundTest {
 
     @Rule
-    public TestRule mockitoRule = new MockitoRule(this);
+    public MethodRule mockitoRule = MockitoJUnit.rule().silent();
 
     @Mock
     private Transformer<Object> first, second;
@@ -37,7 +33,7 @@ public class TransformerCompoundTest {
     }
 
     @Test
-    @SuppressWarnings("unchecked") // In absence of @SafeVarargs for Java 6
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testTransformation() throws Exception {
         assertThat(new Transformer.Compound<Object>(first, second).transform(typeDescription, firstTarget), is(finalTarget));
     }
