@@ -10,30 +10,35 @@ import net.bytebuddy.description.modifier.Visibility;
 import net.bytebuddy.description.type.TypeDefinition;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.Transformer;
+import net.bytebuddy.dynamic.VisibilityBridgeStrategy;
 import net.bytebuddy.implementation.Implementation;
 import net.bytebuddy.implementation.LoadedTypeInitializer;
 import net.bytebuddy.implementation.attribute.MethodAttributeAppender;
 import net.bytebuddy.matcher.ElementMatcher;
 import net.bytebuddy.matcher.ElementMatchers;
 import net.bytebuddy.matcher.LatentMatcher;
-import net.bytebuddy.test.utility.MockitoRule;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.TestRule;
+import org.junit.rules.MethodRule;
 import org.mockito.Mock;
+import org.mockito.junit.MockitoJUnit;
 
 import java.util.Collections;
 
 import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.any;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class MethodRegistryDefaultTest {
 
     @Rule
-    public TestRule mockitoRule = new MockitoRule(this);
+    public MethodRule mockitoRule = MockitoJUnit.rule().silent();
 
     @Mock
     private ClassFileVersion classFileVersion;
@@ -98,6 +103,9 @@ public class MethodRegistryDefaultTest {
     @Mock
     private ParameterDescription.InDefinedShape parameterDescription;
 
+    @Mock
+    private VisibilityBridgeStrategy visibilityBridgeStrategy;
+
     @Before
     @SuppressWarnings("unchecked")
     public void setUp() throws Exception {
@@ -107,7 +115,7 @@ public class MethodRegistryDefaultTest {
         when(secondHandler.compile(implementationTarget)).thenReturn(secondCompiledHandler);
         when(thirdType.getTypeInitializer()).thenReturn(typeInitializer);
         when(thirdType.getLoadedTypeInitializer()).thenReturn(loadedTypeInitializer);
-        when(methodGraphCompiler.compile(thirdType)).thenReturn(methodGraph);
+        when(methodGraphCompiler.compile((TypeDefinition) thirdType)).thenReturn(methodGraph);
         when(methodGraph.listNodes()).thenReturn(new MethodGraph.NodeList(Collections.singletonList(new MethodGraph.Node.Simple(instrumentedMethod))));
         when(firstType.getDeclaredMethods()).thenReturn(new MethodList.Empty<MethodDescription.InDefinedShape>());
         when(secondType.getDeclaredMethods()).thenReturn(new MethodList.Empty<MethodDescription.InDefinedShape>());
@@ -141,7 +149,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Prepared methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter);
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods().size(), is(0));
         assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
@@ -157,7 +165,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Prepared methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter);
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods().size(), is(0));
         assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
@@ -167,14 +175,14 @@ public class MethodRegistryDefaultTest {
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testMatchedFirst() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(firstFilter.matches(instrumentedMethod)).thenReturn(true);
         MethodRegistry.Prepared methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter);
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
         assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
@@ -184,14 +192,14 @@ public class MethodRegistryDefaultTest {
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testMatchedSecond() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(secondFilter.matches(instrumentedMethod)).thenReturn(true);
         MethodRegistry.Prepared methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter);
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
         assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
@@ -201,7 +209,7 @@ public class MethodRegistryDefaultTest {
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testMultipleRegistryDoesNotPrepareMultipleTimes() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(firstFilter.matches(instrumentedMethod)).thenReturn(true);
@@ -215,7 +223,7 @@ public class MethodRegistryDefaultTest {
                 .append(firstMatcher, secondHandler, secondFactory, transformer)
                 .append(firstMatcher, firstHandler, secondFactory, transformer)
                 .append(firstMatcher, secondHandler, firstFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter);
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
         assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
@@ -225,7 +233,7 @@ public class MethodRegistryDefaultTest {
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testCompiledAppendingMatchesFirstAppended() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(firstFilter.matches(instrumentedMethod)).thenReturn(true);
@@ -234,7 +242,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter)
                 .compile(implementationTargetFactory, classFileVersion);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
@@ -243,12 +251,12 @@ public class MethodRegistryDefaultTest {
         verify(firstHandler).prepare(firstType);
         verify(secondHandler).prepare(secondType);
         verify(firstFactory).make(typeDescription);
-        verifyZeroInteractions(secondFactory);
+        verifyNoMoreInteractions(secondFactory);
         assertThat(methodRegistry.target(instrumentedMethod), is(firstRecord));
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testCompiledPrependingMatchesLastPrepended() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(firstFilter.matches(instrumentedMethod)).thenReturn(true);
@@ -257,7 +265,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
                 .prepend(firstMatcher, firstHandler, firstFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter)
                 .compile(implementationTargetFactory, classFileVersion);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
@@ -266,12 +274,12 @@ public class MethodRegistryDefaultTest {
         verify(firstHandler).prepare(firstType);
         verify(secondHandler).prepare(secondType);
         verify(firstFactory).make(typeDescription);
-        verifyZeroInteractions(secondFactory);
+        verifyNoMoreInteractions(secondFactory);
         assertThat(methodRegistry.target(instrumentedMethod), is(firstRecord));
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testCompiledAppendingMatchesSecondAppendedIfFirstDoesNotMatch() throws Exception {
         when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
         when(firstFilter.matches(instrumentedMethod)).thenReturn(false);
@@ -280,7 +288,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter)
                 .compile(implementationTargetFactory, classFileVersion);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods(), is((MethodList) new MethodList.Explicit(instrumentedMethod)));
@@ -288,7 +296,7 @@ public class MethodRegistryDefaultTest {
         assertThat(methodRegistry.getLoadedTypeInitializer(), is(loadedTypeInitializer));
         verify(firstHandler).prepare(firstType);
         verify(secondHandler).prepare(secondType);
-        verifyZeroInteractions(firstFactory);
+        verifyNoMoreInteractions(firstFactory);
         verify(secondFactory).make(typeDescription);
         assertThat(methodRegistry.target(instrumentedMethod), is(secondRecord));
     }
@@ -305,7 +313,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter)
                 .compile(implementationTargetFactory, classFileVersion);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods().size(), is(0));
@@ -313,8 +321,8 @@ public class MethodRegistryDefaultTest {
         assertThat(methodRegistry.getLoadedTypeInitializer(), is(loadedTypeInitializer));
         verify(firstHandler).prepare(firstType);
         verify(secondHandler).prepare(secondType);
-        verifyZeroInteractions(firstFactory);
-        verifyZeroInteractions(secondFactory);
+        verifyNoMoreInteractions(firstFactory);
+        verifyNoMoreInteractions(secondFactory);
         assertThat(methodRegistry.target(instrumentedMethod), instanceOf(TypeWriter.MethodPool.Record.ForNonImplementedMethod.class));
     }
 
@@ -344,7 +352,7 @@ public class MethodRegistryDefaultTest {
         MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
                 .append(firstMatcher, firstHandler, firstFactory, transformer)
                 .append(secondMatcher, secondHandler, secondFactory, transformer)
-                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, methodFilter)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.ALWAYS, methodFilter)
                 .compile(implementationTargetFactory, classFileVersion);
         assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
         assertThat(methodRegistry.getInstrumentedMethods().size(), is(1));
@@ -352,8 +360,47 @@ public class MethodRegistryDefaultTest {
         assertThat(methodRegistry.getLoadedTypeInitializer(), is(loadedTypeInitializer));
         verify(firstHandler).prepare(firstType);
         verify(secondHandler).prepare(secondType);
-        verifyZeroInteractions(firstFactory);
-        verifyZeroInteractions(secondFactory);
+        verifyNoMoreInteractions(firstFactory);
+        verifyNoMoreInteractions(secondFactory);
         assertThat(methodRegistry.target(instrumentedMethod), instanceOf(TypeWriter.MethodPool.Record.ForDefinedMethod.OfVisibilityBridge.class));
     }
+
+    @Test
+    @SuppressWarnings("unchecked")
+    public void testVisibilityBridgeIfNotMatchedAndVisibleBridgesDisabled() throws Exception {
+        when(instrumentedMethod.getDeclaredAnnotations()).thenReturn(new AnnotationList.Empty());
+        when(parameterDescription.getDeclaredAnnotations()).thenReturn(new AnnotationList.Empty());
+        when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
+        when(firstFilter.matches(instrumentedMethod)).thenReturn(false);
+        when(secondFilter.matches(instrumentedMethod)).thenReturn(false);
+        when(resolvedMethodFilter.matches(instrumentedMethod)).thenReturn(true);
+        TypeDescription declaringType = mock(TypeDescription.class);
+        when(declaringType.asErasure()).thenReturn(declaringType);
+        when(instrumentedMethod.getDeclaringType()).thenReturn(declaringType);
+        when(thirdType.isPublic()).thenReturn(true);
+        when(instrumentedMethod.isPublic()).thenReturn(true);
+        when(declaringType.isPackagePrivate()).thenReturn(true);
+        TypeDescription.Generic superClass = mock(TypeDescription.Generic.class);
+        TypeDescription rawSuperClass = mock(TypeDescription.class);
+        when(superClass.asErasure()).thenReturn(rawSuperClass);
+        when(typeDescription.getSuperClass()).thenReturn(superClass);
+        MethodDescription.Token methodToken = mock(MethodDescription.Token.class);
+        when(instrumentedMethod.asToken(ElementMatchers.is(typeDescription))).thenReturn(methodToken);
+        when(methodToken.accept(any(TypeDescription.Generic.Visitor.class))).thenReturn(methodToken);
+        when(classFileVersion.isAtLeast(ClassFileVersion.JAVA_V5)).thenReturn(true);
+        MethodRegistry.Compiled methodRegistry = new MethodRegistry.Default()
+                .append(firstMatcher, firstHandler, firstFactory, transformer)
+                .append(secondMatcher, secondHandler, secondFactory, transformer)
+                .prepare(firstType, methodGraphCompiler, TypeValidation.ENABLED, VisibilityBridgeStrategy.Default.NEVER, methodFilter)
+                .compile(implementationTargetFactory, classFileVersion);
+        assertThat(methodRegistry.getInstrumentedType(), is(typeDescription));
+        assertThat(methodRegistry.getInstrumentedMethods().size(), is(0));
+        assertThat(methodRegistry.getTypeInitializer(), is(typeInitializer));
+        assertThat(methodRegistry.getLoadedTypeInitializer(), is(loadedTypeInitializer));
+        verify(firstHandler).prepare(firstType);
+        verify(secondHandler).prepare(secondType);
+        verifyNoMoreInteractions(firstFactory);
+        verifyNoMoreInteractions(secondFactory);
+        assertThat(methodRegistry.target(instrumentedMethod), instanceOf(TypeWriter.MethodPool.Record.ForDefinedMethod.ForNonImplementedMethod.class));
+    }
 }
