@@ -3,59 +3,80 @@ package net.bytebuddy.dynamic.scaffold.subclass;
 import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.description.annotation.AnnotationDescription;
 import net.bytebuddy.description.method.MethodDescription;
+import net.bytebuddy.description.modifier.ModifierContributor;
 import net.bytebuddy.description.modifier.Visibility;
+import net.bytebuddy.description.module.ModuleDescription;
 import net.bytebuddy.description.type.PackageDescription;
+import net.bytebuddy.description.type.TypeDefinition;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.AbstractDynamicTypeBuilderTest;
+import net.bytebuddy.dynamic.ClassFileLocator;
 import net.bytebuddy.dynamic.DynamicType;
 import net.bytebuddy.dynamic.TargetType;
 import net.bytebuddy.dynamic.loading.ByteArrayClassLoader;
 import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
+import net.bytebuddy.dynamic.loading.ModuleLayerFromSingleClassLoaderDecorator;
 import net.bytebuddy.dynamic.scaffold.TypeValidation;
 import net.bytebuddy.implementation.FixedValue;
 import net.bytebuddy.implementation.Implementation;
 import net.bytebuddy.implementation.MethodDelegation;
 import net.bytebuddy.implementation.StubMethod;
+import net.bytebuddy.implementation.SuperMethodCall;
 import net.bytebuddy.implementation.bytecode.constant.TextConstant;
 import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.test.scope.GenericType;
-import net.bytebuddy.test.utility.ClassFileExtraction;
 import net.bytebuddy.test.utility.InjectionStrategyResolver;
 import net.bytebuddy.test.utility.JavaVersionRule;
 import org.hamcrest.CoreMatchers;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.MethodRule;
+import org.objectweb.asm.ClassReader;
 import org.objectweb.asm.Opcodes;
+import org.objectweb.asm.util.ASMifier;
+import org.objectweb.asm.util.Textifier;
+import org.objectweb.asm.util.TraceClassVisitor;
 
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.PrintWriter;
 import java.lang.annotation.Annotation;
+import java.lang.annotation.ElementType;
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
+import java.lang.reflect.AnnotatedElement;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.Type;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
 import static junit.framework.TestCase.assertEquals;
 import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
 import static net.bytebuddy.matcher.ElementMatchers.named;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.nullValue;
+import static org.hamcrest.CoreMatchers.sameInstance;
 import static org.hamcrest.MatcherAssert.assertThat;
 
 public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTest {
 
-    private static final String TYPE_VARIABLE_NAME = "net.bytebuddy.test.precompiled.TypeAnnotation", VALUE = "value";
+    private static final String TYPE_VARIABLE_NAME = "net.bytebuddy.test.precompiled.v8.TypeAnnotation", VALUE = "value";
 
     private static final String FOO = "foo", BAR = "bar", QUX = "qux";
 
     private static final int BAZ = 42;
 
-    private static final String DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.SingleDefaultMethodInterface";
+    private static final String DEFAULT_METHOD_INTERFACE = "net.bytebuddy.test.precompiled.v8.SingleDefaultMethodInterface";
 
-    private static final String PARAMETER_NAME_CLASS = "net.bytebuddy.test.precompiled.ParameterNames";
+    private static final String PARAMETER_NAME_CLASS = "net.bytebuddy.test.precompiled.v8parameters.ParameterNames";
 
     private static final Object STATIC_FIELD = null;
 
@@ -64,12 +85,14 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
     @Rule
     public MethodRule javaVersionRule = new JavaVersionRule();
 
-    @Override
     protected DynamicType.Builder<?> createPlain() {
         return new ByteBuddy().subclass(Object.class);
     }
 
-    @Override
+    protected DynamicType.Builder<?> createPlainEmpty() {
+        return createPlain();
+    }
+
     protected DynamicType.Builder<?> createPlainWithoutValidation() {
         return new ByteBuddy().with(TypeValidation.DISABLED).subclass(Object.class);
     }
@@ -171,7 +194,7 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
     }
 
     @Test
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public void testEnumerationDefinition() throws Exception {
         Class<? extends Enum<?>> type = new ByteBuddy()
                 .makeEnumeration(FOO, BAR)
@@ -186,10 +209,10 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
         assertThat(type.isInterface(), is(false));
         assertThat(type.isAnnotation(), is(false));
         assertThat(type.isEnum(), is(true));
-        Enum foo = Enum.valueOf((Class) type, FOO);
+        Enum<?> foo = Enum.valueOf((Class) type, FOO);
         assertThat(foo.name(), is(FOO));
         assertThat(foo.ordinal(), is(0));
-        Enum bar = Enum.valueOf((Class) type, BAR);
+        Enum<?> bar = Enum.valueOf((Class) type, BAR);
         assertThat(bar.name(), is(BAR));
         assertThat(bar.ordinal(), is(1));
     }
@@ -211,6 +234,66 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
         assertThat(packageType.getAnnotation(Foo.class), notNullValue(Foo.class));
     }
 
+    @Test
+    @JavaVersionRule.Enforce(9)
+    public void testModuleDefinition() throws Exception {
+        DynamicType.Unloaded<? extends Annotation> annotation = new ByteBuddy()
+                .makeAnnotation()
+                .name(BAR + "." + FOO)
+                .annotateType(AnnotationDescription.Builder.ofType(Retention.class)
+                        .define("value", RetentionPolicy.RUNTIME)
+                        .build())
+                .annotateType(AnnotationDescription.Builder.ofType(Target.class)
+                        .defineEnumerationArray("value", ElementType.class, ElementType.valueOf("MODULE"))
+                        .build())
+                .make();
+        Class<?> type = new ByteBuddy()
+                .subclass(Object.class)
+                .name(BAR + "." + QUX)
+                .make()
+                .include(new ByteBuddy()
+                        .makeModule(FOO)
+                        .version("1")
+                        .mainClass(BAR + "." + QUX)
+                        .packages(BAR)
+                        .require("java.instrument")
+                        .export(BAR)
+                        .open(BAR)
+                        .uses(Runnable.class)
+                        .provides(Runnable.class.getName(), BAR + "." + QUX)
+                        .annotateType(AnnotationDescription.Builder.ofType(annotation.getTypeDescription()).build())
+                        .make(), annotation)
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER.with(ModuleLayerFromSingleClassLoaderDecorator.Factory.INSTANCE))
+                .getLoaded();
+        ModuleDescription moduleDescription = ModuleDescription.ForLoadedModule.of(Class.class.getMethod("getModule").invoke(type));
+        assertThat(moduleDescription.getActualName(), is(FOO));
+        assertThat(moduleDescription.getModifiers(), is(ModifierContributor.EMPTY_MASK));
+        assertThat(moduleDescription.getVersion(), is("1"));
+        assertThat(moduleDescription.getMainClass(), is(BAR + "." + QUX));
+        assertThat(moduleDescription.getRequires().size(), is(2));
+        assertThat(moduleDescription.getRequires().get("java.base"), notNullValue(ModuleDescription.Requires.class));
+        assertThat(moduleDescription.getRequires().get("java.base").getModifiers(), is(Opcodes.ACC_MANDATED));
+        assertThat(moduleDescription.getRequires().get("java.base").getVersion(), nullValue(String.class));
+        assertThat(moduleDescription.getRequires().get("java.instrument"), notNullValue(ModuleDescription.Requires.class));
+        assertThat(moduleDescription.getRequires().get("java.instrument").getModifiers(), is(0));
+        assertThat(moduleDescription.getRequires().get("java.instrument").getVersion(), nullValue(String.class));
+        assertThat(moduleDescription.getExports().size(), is(1));
+        assertThat(moduleDescription.getExports().get(BAR), notNullValue(ModuleDescription.Exports.class));
+        assertThat(moduleDescription.getExports().get(BAR).getModifiers(), is(0));
+        assertThat(moduleDescription.getExports().get(BAR).getTargets().size(), is(0));
+        assertThat(moduleDescription.getOpens().size(), is(1));
+        assertThat(moduleDescription.getOpens().get(BAR), notNullValue(ModuleDescription.Opens.class));
+        assertThat(moduleDescription.getOpens().get(BAR).getModifiers(), is(0));
+        assertThat(moduleDescription.getOpens().get(BAR).getTargets().size(), is(0));
+        assertThat(moduleDescription.getUses().size(), is(1));
+        assertThat(moduleDescription.getUses().contains(Runnable.class.getName()), is(true));
+        assertThat(moduleDescription.getProvides().size(), is(1));
+        assertThat(moduleDescription.getProvides().get(Runnable.class.getName()), notNullValue(ModuleDescription.Provides.class));
+        assertThat(moduleDescription.getProvides().get(Runnable.class.getName()).getProviders(), is(Collections.singleton(BAR + "." + QUX)));
+        // Annotations are currently not visible from dynamic layers as they are resolved via URIs, and not through the ModuleReader.
+        assertThat(moduleDescription.getDeclaredAnnotations().size(), is(0));
+    }
+
     @Test
     @JavaVersionRule.Enforce(8)
     public void testDefaultMethodNonOverridden() throws Exception {
@@ -297,8 +380,8 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .method(isDeclaredBy(PackagePrivateReturnType.class))
                 .intercept(StubMethod.INSTANCE)
                 .make()
-                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER,
-                        ClassFileExtraction.of(PackagePrivateReturnType.class, PackagePrivateReturnType.Argument.class)), ClassLoadingStrategy.Default.WRAPPER)
+                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassFileLocator.ForClassLoader.readToNames(PackagePrivateReturnType.class,
+                        PackagePrivateReturnType.Argument.class)), ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded();
         assertThat(type.getDeclaredMethods().length, is(0));
     }
@@ -311,8 +394,8 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .method(isDeclaredBy(PackagePrivateArgumentType.class))
                 .intercept(StubMethod.INSTANCE)
                 .make()
-                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER,
-                        ClassFileExtraction.of(PackagePrivateArgumentType.class, PackagePrivateArgumentType.Argument.class)), ClassLoadingStrategy.Default.WRAPPER)
+                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassFileLocator.ForClassLoader.readToNames(PackagePrivateArgumentType.class,
+                        PackagePrivateArgumentType.Argument.class)), ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded();
         assertThat(type.getDeclaredMethods().length, is(0));
     }
@@ -324,8 +407,8 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .method(isDeclaredBy(PrivateMethod.class))
                 .intercept(StubMethod.INSTANCE)
                 .make()
-                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER,
-                        ClassFileExtraction.of(PrivateMethod.class)), ClassLoadingStrategy.Default.WRAPPER)
+                .load(new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassFileLocator.ForClassLoader.readToNames(PrivateMethod.class)),
+                        ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded();
         assertThat(type.getDeclaredMethods().length, is(0));
     }
@@ -504,8 +587,8 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .load(typeAnnotationType.getClassLoader(), ClassLoadingStrategy.Default.CHILD_FIRST)
                 .getLoaded();
         assertThat(type.getSuperclass(), is((Object) Object.class));
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveSuperClassType(type).asList().size(), is(1));
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveSuperClassType(type).asList().ofType(typeAnnotationType)
+        assertThat(new TypeDescription.Generic.AnnotationReader.Delegator.ForLoadedSuperClass(type).asList().size(), is(1));
+        assertThat(new TypeDescription.Generic.AnnotationReader.Delegator.ForLoadedSuperClass(type).asList().ofType(typeAnnotationType)
                 .getValue(value).resolve(Integer.class), is(BAZ));
     }
 
@@ -525,9 +608,9 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .load(typeAnnotationType.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded()
                 .getDeclaredMethod(FOO);
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveReceiverType(method).getDeclaredAnnotations().size(), is(1));
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveReceiverType(method).getDeclaredAnnotations()
-                .ofType(typeAnnotationType).getValue(value).resolve(Integer.class), is(BAZ));
+        TypeDescription.Generic receiver = TypeDefinition.Sort.describeAnnotated((AnnotatedElement) Method.class.getMethod("getAnnotatedReceiverType").invoke(method));
+        assertThat(receiver.getDeclaredAnnotations().size(), is(1));
+        assertThat(receiver.getDeclaredAnnotations().ofType(typeAnnotationType).getValue(value).resolve(Integer.class), is(BAZ));
     }
 
     @Test
@@ -546,9 +629,19 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
                 .load(typeAnnotationType.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER)
                 .getLoaded()
                 .getDeclaredMethod("toString");
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveReceiverType(method).getDeclaredAnnotations().size(), is(1));
-        assertThat(TypeDescription.Generic.AnnotationReader.DISPATCHER.resolveReceiverType(method).getDeclaredAnnotations()
-                .ofType(typeAnnotationType).getValue(value).resolve(Integer.class), is(BAZ));
+        TypeDescription.Generic receiver = TypeDefinition.Sort.describeAnnotated((AnnotatedElement) Method.class.getMethod("getAnnotatedReceiverType").invoke(method));
+        assertThat(receiver.getDeclaredAnnotations().size(), is(1));
+        assertThat(receiver.getDeclaredAnnotations().ofType(typeAnnotationType).getValue(value).resolve(Integer.class), is(BAZ));
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testBridgeMethodExplicit() throws Exception {
+        new ByteBuddy()
+                .subclass(GenericBase.Subclass.class)
+                .defineMethod("foo", void.class, Modifier.PUBLIC)
+                .withParameters(Object.class)
+                .intercept(SuperMethodCall.INSTANCE)
+                .make();
     }
 
     @SuppressWarnings("unused")
@@ -652,4 +745,17 @@ public class SubclassDynamicTypeBuilderTest extends AbstractDynamicTypeBuilderTe
             /* empty */
         }
     }
+
+    public static abstract class GenericBase<T> {
+
+        public abstract void foo(T argument);
+
+        public static class Subclass extends GenericBase<String> {
+
+            @Override
+            public void foo(String argument) {
+                /* empty */
+            }
+        }
+    }
 }
