@@ -7,6 +7,8 @@ import net.bytebuddy.description.modifier.Visibility;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.DynamicType;
 import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
+import net.bytebuddy.dynamic.scaffold.FieldLocator;
+import net.bytebuddy.dynamic.scaffold.subclass.ConstructorStrategy;
 import net.bytebuddy.implementation.bytecode.StackManipulation;
 import net.bytebuddy.implementation.bytecode.assign.Assigner;
 import net.bytebuddy.implementation.bytecode.constant.TextConstant;
@@ -14,7 +16,6 @@ import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.matcher.ElementMatchers;
 import net.bytebuddy.test.utility.CallTraceable;
 import net.bytebuddy.test.utility.JavaVersionRule;
-import net.bytebuddy.test.utility.MockitoRule;
 import net.bytebuddy.utility.JavaConstant;
 import net.bytebuddy.utility.JavaType;
 import org.hamcrest.CoreMatchers;
@@ -22,28 +23,40 @@ import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.MethodRule;
-import org.junit.rules.TestRule;
 import org.mockito.Mock;
 import org.mockito.Mockito;
+import org.mockito.junit.MockitoJUnit;
 
 import java.lang.reflect.Method;
 import java.util.concurrent.Callable;
 
-import static net.bytebuddy.matcher.ElementMatchers.*;
-import static org.hamcrest.CoreMatchers.*;
+import static net.bytebuddy.matcher.ElementMatchers.isConstructor;
+import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
+import static net.bytebuddy.matcher.ElementMatchers.isEquals;
+import static net.bytebuddy.matcher.ElementMatchers.isTypeInitializer;
+import static net.bytebuddy.matcher.ElementMatchers.named;
+import static net.bytebuddy.matcher.ElementMatchers.takesArguments;
+import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.nullValue;
+import static org.hamcrest.CoreMatchers.sameInstance;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class MethodCallTest {
 
-    private static final String FOO = "foo", BAR = "bar", INVOKE_FOO = "invokeFoo";
+    private static final String FOO = "foo", BAR = "bar", QUX = "qux", INVOKE_FOO = "invokeFoo";
 
-    private static final String SINGLE_DEFAULT_METHOD = "net.bytebuddy.test.precompiled.SingleDefaultMethodInterface";
+    private static final String SINGLE_DEFAULT_METHOD = "net.bytebuddy.test.precompiled.v8.SingleDefaultMethodInterface";
 
     @Rule
-    public TestRule methodRule = new MockitoRule(this);
+    public MethodRule mockitoRule = MockitoJUnit.rule().silent();
 
     @Rule
     public MethodRule javaVersionRule = new JavaVersionRule();
@@ -136,6 +149,23 @@ public class MethodCallTest {
         assertThat(instance, instanceOf(InstanceMethod.class));
     }
 
+    @Test
+    public void testMethodInvocationUsingStackManipulation() throws Exception {
+        DynamicType.Loaded<SimpleMethod> loaded = new ByteBuddy()
+                .subclass(SimpleMethod.class)
+                .method(named(FOO))
+                .intercept(MethodCall.invoke(String.class.getMethod("toUpperCase")).on(new TextConstant(FOO), String.class))
+                .make()
+                .load(SimpleMethod.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        SimpleMethod instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.foo(), is(FOO.toUpperCase()));
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(SimpleMethod.class)));
+        assertThat(instance, instanceOf(SimpleMethod.class));
+    }
+
     @Test(expected = IllegalStateException.class)
     public void testMatchedCallAmbiguous() throws Exception {
         new ByteBuddy()
@@ -176,6 +206,23 @@ public class MethodCallTest {
         assertThat(instance, instanceOf(ArgumentCall.class));
     }
 
+    @Test
+    public void testInvokeOnArgumentUsingMatcher() throws Exception {
+        DynamicType.Loaded<ArgumentCall> loaded = new ByteBuddy()
+                .subclass(ArgumentCall.class)
+                .method(named("foo"))
+                .intercept(MethodCall.invoke(named("toUpperCase").and(takesArguments(0))).onMethodCall(MethodCall.invoke(named("foo")).onArgument(0)))
+                .make()
+                .load(ArgumentCall.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        ArgumentCall instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.foo(new ArgumentCall.Target("foo")), is("FOO"));
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(InstanceMethod.class)));
+        assertThat(instance, instanceOf(ArgumentCall.class));
+    }
+
     @Test(expected = IllegalStateException.class)
     public void testInvokeOnArgumentNonAssignable() throws Exception {
         new ByteBuddy()
@@ -212,6 +259,122 @@ public class MethodCallTest {
         assertThat(instance, instanceOf(ArgumentCallDynamic.class));
     }
 
+    @Test
+    public void testInvokeOnMethodCall() throws Exception {
+        DynamicType.Loaded<MethodCallChaining> loaded = new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .method(named("foobar"))
+                .intercept(MethodCall.invoke(String.class.getMethod("toUpperCase"))
+                        .onMethodCall(MethodCall.invoke(named("bar"))))
+                .make()
+                .load(MethodCallChaining.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        MethodCallChaining instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.foobar(), is("BAR"));
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(InstanceMethod.class)));
+        assertThat(instance, instanceOf(MethodCallChaining.class));
+    }
+
+    @Test
+    public void testInvokeOnMethodCallUsingMatcher() throws Exception {
+        DynamicType.Loaded<MethodCallChaining> loaded = new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .method(named("foobar"))
+                .intercept(MethodCall.invoke(named("toUpperCase").and(takesArguments(0)))
+                        .onMethodCall(MethodCall.invoke(named("bar"))))
+                .make()
+                .load(MethodCallChaining.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        MethodCallChaining instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.foobar(), is("BAR"));
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(InstanceMethod.class)));
+        assertThat(instance, instanceOf(MethodCallChaining.class));
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testInvokeOnIncompatibleMethodCall() throws Exception {
+        new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .method(named("foobar"))
+                .intercept(MethodCall.invoke(Integer.class.getMethod("toString"))
+                        .onMethodCall(MethodCall.invoke(named("bar"))))
+                .make();
+    }
+
+    @Test
+    public void testStaticOnStaticFieldFromAnotherClass() throws Exception {
+        DynamicType.Loaded<Object> loaded = new ByteBuddy()
+                .subclass(Object.class)
+                .invokable(isTypeInitializer())
+                .intercept(MethodCall.invoke(isEquals())
+                        .onField("out", new FieldLocator.ForExactType.Factory(TypeDescription.ForLoadedType.of(System.class)))
+                        .with(""))
+                .make()
+                .load(Object.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(0));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        Object instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance, instanceOf(Object.class));
+    }
+
+    @Test
+    public void testStaticFieldExplicit() throws Exception {
+        DynamicType.Loaded<Object> loaded = new ByteBuddy()
+                .subclass(Object.class)
+                .invokable(isTypeInitializer())
+                .intercept(MethodCall.invoke(isEquals())
+                        .onField(System.class.getField("out"))
+                        .with(""))
+                .make()
+                .load(Object.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(0));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        Object instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance, instanceOf(Object.class));
+    }
+
+    @Test
+    public void testOnStaticFieldFromAnotherClass() throws Exception {
+        DynamicType.Loaded<Object> loaded = new ByteBuddy()
+                .subclass(Object.class)
+                .defineMethod("foo", void.class)
+                .intercept(MethodCall.invoke(isEquals())
+                        .onField("out", new FieldLocator.ForExactType.Factory(TypeDescription.ForLoadedType.of(System.class)))
+                        .with("fooCall"))
+                .make()
+                .load(Object.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        Object instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance, instanceOf(Object.class));
+    }
+
+    @Test
+    public void testOnFieldUsingMatcher() throws Exception {
+        DynamicType.Loaded<MethodCallOnField> loaded = new ByteBuddy()
+                .subclass(MethodCallOnField.class)
+                .method(named("foo"))
+                .intercept(MethodCall.invoke(named("trim")).onField("foo"))
+                .make()
+                .load(MethodCallOnField.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredMethod(FOO), not(nullValue(Method.class)));
+        assertThat(loaded.getLoaded().getDeclaredConstructors().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        MethodCallOnField instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        instance.foo = " abc ";
+        assertThat(instance, instanceOf(MethodCallOnField.class));
+        assertThat(instance.foo(), is("abc"));
+    }
+
     @Test
     public void testSuperConstructorInvocationWithoutArguments() throws Exception {
         DynamicType.Loaded<Object> loaded = new ByteBuddy()
@@ -229,6 +392,19 @@ public class MethodCallTest {
         assertThat(instance, instanceOf(Object.class));
     }
 
+    @Test
+    public void testSuperConstructorInvocationUsingMatcher() throws Exception {
+        assertThat(new ByteBuddy()
+                .subclass(Object.class, ConstructorStrategy.Default.NO_CONSTRUCTORS)
+                .defineConstructor(Visibility.PUBLIC)
+                .intercept(MethodCall.invoke(isConstructor().and(isDeclaredBy(Object.class))).onSuper())
+                .make()
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER)
+                .getLoaded()
+                .getConstructor()
+                .newInstance(), notNullValue(Object.class));
+    }
+
     @Test
     public void testObjectConstruction() throws Exception {
         DynamicType.Loaded<SelfReference> loaded = new ByteBuddy()
@@ -272,6 +448,29 @@ public class MethodCallTest {
         verifyNoMoreInteractions(delegate);
     }
 
+    @Test
+    public void testSelfInvocationViaMatcher() throws Exception {
+        SuperMethodInvocation delegate = mock(SuperMethodInvocation.class);
+        when(delegate.foo()).thenReturn(FOO);
+        DynamicType.Loaded<SuperMethodInvocation> loaded = new ByteBuddy()
+                .subclass(SuperMethodInvocation.class)
+                .method(takesArguments(0).and(named(FOO)))
+                .intercept(MethodCall.invoke(takesArguments(0).and(named(FOO))).on(delegate, SuperMethodInvocation.class))
+                .make()
+                .load(SuperMethodInvocation.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredMethod(FOO), not(nullValue(Method.class)));
+        assertThat(loaded.getLoaded().getDeclaredConstructors().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(1));
+        SuperMethodInvocation instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(SuperMethodInvocation.class)));
+        assertThat(instance, instanceOf(SuperMethodInvocation.class));
+        assertThat(instance.foo(), is(FOO));
+        verify(delegate).foo();
+        verifyNoMoreInteractions(delegate);
+    }
+
     @Test
     public void testSuperInvocation() throws Exception {
         DynamicType.Loaded<SuperMethodInvocation> loaded = new ByteBuddy()
@@ -675,6 +874,41 @@ public class MethodCallTest {
         assertThat(instance.foo(null), CoreMatchers.<Class<?>>is(loaded.getLoaded()));
     }
 
+    @Test
+    public void testInvokeWithMethodCall() throws Exception {
+        DynamicType.Loaded<MethodCallChaining> loaded = new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .method(named("foobar"))
+                .intercept(MethodCall.invoke(named("foo")).withMethodCall(MethodCall.invoke(named("bar"))))
+                .make()
+                .load(MethodCallChaining.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        MethodCallChaining instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredMethod("foobar"), not(nullValue(Method.class)));
+        assertThat(loaded.getLoaded().getDeclaredConstructors().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        assertThat(instance.foobar(), is("foobar"));
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testStaticInvokeWithMethodCall() throws Exception {
+        new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .defineMethod("staticFoobar", String.class, Ownership.STATIC)
+                .intercept(MethodCall.invoke(named("foo")).withMethodCall(MethodCall.invoke(named("bar"))))
+                .make();
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testInvokeWithIncompatibleMethodCall() throws Exception {
+        new ByteBuddy()
+                .subclass(MethodCallChaining.class)
+                .method(named("foobar"))
+                .intercept(MethodCall.invoke(named("foo")).withMethodCall(MethodCall.invoke(named("someInt"))))
+                .make();
+    }
+
     @Test
     public void testImplementationAppendingMethod() throws Exception {
         DynamicType.Loaded<MethodCallAppending> loaded = new ByteBuddy()
@@ -763,7 +997,7 @@ public class MethodCallTest {
         DynamicType.Loaded<SimpleMethod> loaded = new ByteBuddy()
                 .subclass(SimpleMethod.class)
                 .method(named(FOO))
-                .intercept(MethodCall.invoke(Foo.class.getDeclaredMethod(BAR, Object.class, Object.class)).with(TypeDescription.OBJECT, TypeDescription.STRING))
+                .intercept(MethodCall.invoke(Foo.class.getDeclaredMethod(BAR, Object.class, Object.class)).with(TypeDescription.ForLoadedType.of(Object.class), TypeDescription.ForLoadedType.of(String.class)))
                 .make()
                 .load(SimpleMethod.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
         assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
@@ -776,7 +1010,7 @@ public class MethodCallTest {
     }
 
     @Test
-    @JavaVersionRule.Enforce(value = 7, hotSpot = 7)
+    @JavaVersionRule.Enforce(value = 7, atMost = 7, j9 = false)
     public void testJava7Types() throws Exception {
         DynamicType.Loaded<SimpleMethod> loaded = new ByteBuddy()
                 .subclass(SimpleMethod.class)
@@ -794,7 +1028,7 @@ public class MethodCallTest {
     }
 
     @Test
-    @JavaVersionRule.Enforce(value = 7, hotSpot = 7)
+    @JavaVersionRule.Enforce(value = 7, atMost = 7, j9 = false)
     public void testJava7TypesExplicit() throws Exception {
         DynamicType.Loaded<SimpleMethod> loaded = new ByteBuddy()
                 .subclass(SimpleMethod.class)
@@ -854,6 +1088,70 @@ public class MethodCallTest {
         traceable.assertOnlyCall(FOO);
     }
 
+    @Test
+    public void testFieldSetting() throws Exception {
+        DynamicType.Loaded<FieldSetting> loaded = new ByteBuddy()
+                .subclass(FieldSetting.class)
+                .method(named(FOO))
+                .intercept(MethodCall.invoke(FieldSetting.class.getMethod(BAR)).setsField(FieldSetting.class.getField(FOO)))
+                .make()
+                .load(FieldSetting.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        FieldSetting instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        instance.foo();
+        assertThat(instance.foo, is((Object) instance.bar()));
+    }
+
+    @Test
+    public void testFieldSettingWithMatcher() throws Exception {
+        DynamicType.Loaded<FieldSetting> loaded = new ByteBuddy()
+                .subclass(FieldSetting.class)
+                .method(named(FOO))
+                .intercept(MethodCall.invoke(FieldSetting.class.getMethod(BAR)).setsField(ElementMatchers.is(FieldSetting.class.getField(FOO))))
+                .make()
+                .load(FieldSetting.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        FieldSetting instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        instance.foo();
+        assertThat(instance.foo, is((Object) instance.bar()));
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testFieldSettingLast() throws Exception {
+        new ByteBuddy()
+                .subclass(FieldSetting.class)
+                .method(named(QUX))
+                .intercept(MethodCall.invoke(FieldSetting.class.getMethod(BAR)).setsField(FieldSetting.class.getField(FOO)))
+                .make();
+    }
+
+    @Test
+    public void testFieldSettingLastChained() throws Exception {
+        DynamicType.Loaded<FieldSetting> loaded = new ByteBuddy()
+                .subclass(FieldSetting.class)
+                .method(named(QUX))
+                .intercept(MethodCall.invoke(FieldSetting.class.getMethod(BAR))
+                        .setsField(ElementMatchers.is(FieldSetting.class.getField(FOO)))
+                        .andThen(StubMethod.INSTANCE))
+                .make()
+                .load(FieldSetting.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        FieldSetting instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.qux(), nullValue(Object.class));
+        assertThat(instance.foo, is((Object) instance.bar()));
+    }
+
+    @Test
+    public void testConstructorIsAccessibleFromDifferentPackage() throws Exception {
+        assertThat(new ByteBuddy()
+                .subclass(ProtectedConstructor.class, ConstructorStrategy.Default.NO_CONSTRUCTORS)
+                .name("foo.Bar")
+                .defineConstructor(Visibility.PUBLIC)
+                .intercept(MethodCall.invoke(ProtectedConstructor.class.getDeclaredConstructor()).onSuper())
+                .make()
+                .load(Foo.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER)
+                .getLoaded()
+                .getConstructor()
+                .newInstance(), instanceOf(ProtectedConstructor.class));
+    }
+
     @Test(expected = IllegalStateException.class)
     public void testDefaultMethodNotCompatible() throws Exception {
         new ByteBuddy()
@@ -993,6 +1291,84 @@ public class MethodCallTest {
                 .make();
     }
 
+    @Test(expected = IllegalStateException.class)
+    public void testCallConstructorFromMethod() throws Exception {
+        new ByteBuddy()
+                .subclass(Object.class)
+                .defineMethod(FOO, void.class)
+                .intercept(MethodCall.invoke(Object.class.getConstructor()))
+                .make();
+    }
+
+    @Test(expected = IllegalStateException.class)
+    public void testConstructorCallFromNonRelatedConstructor() throws Exception {
+        new ByteBuddy()
+                .subclass(Number.class)
+                .constructor(ElementMatchers.any())
+                .intercept(MethodCall.invoke(Object.class.getConstructor()))
+                .make();
+    }
+
+    @Test
+    public void testConstructorCallResultIntoMethod() throws Exception {
+        DynamicType.Loaded<ConstructorResult> loaded = new ByteBuddy()
+                .subclass(ConstructorResult.class)
+                .method(ElementMatchers.named(FOO))
+                .intercept(MethodCall.invoke(ElementMatchers.named("method"))
+                        .withMethodCall(MethodCall.construct(ConstructorResult.Target.class.getConstructor())))
+                .make()
+                .load(ConstructorResult.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredMethod(FOO), not(nullValue(Method.class)));
+        assertThat(loaded.getLoaded().getDeclaredConstructors().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        ConstructorResult instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(ConstructorResult.class)));
+        assertThat(instance, instanceOf(ConstructorResult.class));
+        assertThat(instance.foo(), instanceOf(ConstructorResult.Target.class));
+    }
+
+    @Test
+    public void testConstructorCallResultIntoField() throws Exception {
+        DynamicType.Loaded<ConstructorResult> loaded = new ByteBuddy()
+                .subclass(ConstructorResult.class)
+                .method(ElementMatchers.named(FOO))
+                .intercept(MethodCall.construct(ConstructorResult.Target.class.getConstructor())
+                        .setsField(ElementMatchers.named("field"))
+                        .andThen(FieldAccessor.ofField("field")))
+                .make()
+                .load(ConstructorResult.class.getClassLoader(), ClassLoadingStrategy.Default.WRAPPER);
+        assertThat(loaded.getLoadedAuxiliaryTypes().size(), is(0));
+        assertThat(loaded.getLoaded().getDeclaredMethods().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredMethod(FOO), not(nullValue(Method.class)));
+        assertThat(loaded.getLoaded().getDeclaredConstructors().length, is(1));
+        assertThat(loaded.getLoaded().getDeclaredFields().length, is(0));
+        ConstructorResult instance = loaded.getLoaded().getDeclaredConstructor().newInstance();
+        assertThat(instance.getClass(), not(CoreMatchers.<Class<?>>is(ConstructorResult.class)));
+        assertThat(instance, instanceOf(ConstructorResult.class));
+        ConstructorResult.Target foo = instance.foo();
+        assertThat(foo, instanceOf(ConstructorResult.Target.class));
+        assertThat(instance.field, equalTo(foo));
+    }
+
+    public static class ConstructorResult {
+
+        public Target field;
+
+        public Target method(Target args) {
+            return args;
+        }
+
+        public Target foo() {
+            return null;
+        }
+
+        public static class Target {
+
+        }
+    }
+
     public static class SimpleMethod {
 
         public String foo() {
@@ -1106,6 +1482,15 @@ public class MethodCallTest {
         }
     }
 
+    public static class MethodCallOnField {
+
+        public String foo;
+
+        public String foo() {
+            return foo;
+        }
+    }
+
     @SuppressWarnings("unused")
     public static class InvisibleMethodCallWithField extends InvisibleBase {
 
@@ -1135,6 +1520,23 @@ public class MethodCallTest {
         }
     }
 
+    public static abstract class MethodCallChaining {
+
+        public String foo(String bar) {
+            return "foo" + bar;
+        }
+
+        public String bar() {
+            return "bar";
+        }
+
+        public abstract String foobar();
+
+        public int someInt() {
+            return 0xCAFEBABE;
+        }
+    }
+
     public static class MethodCallAppending extends CallTraceable {
 
         public Object foo() {
@@ -1192,13 +1594,11 @@ public class MethodCallTest {
 
     public static class Traceable extends CallTraceable implements Runnable, Callable<String> {
 
-        @Override
         public String call() throws Exception {
             register(FOO);
             return FOO;
         }
 
-        @Override
         public void run() {
             register(FOO);
         }
@@ -1219,4 +1619,28 @@ public class MethodCallTest {
             return arg[0] + arg[1];
         }
     }
+
+    public static class FieldSetting {
+
+        public String foo;
+
+        public void foo() {
+            throw new AssertionError();
+        }
+
+        public String bar() {
+            return FOO;
+        }
+
+        public String qux() {
+            throw new AssertionError();
+        }
+    }
+
+    public static class ProtectedConstructor {
+
+        protected ProtectedConstructor() {
+            /* empty */
+        }
+    }
 }
