@@ -4,7 +4,9 @@ import org.junit.Before;
 import org.junit.Test;
 import org.mockito.Mock;
 
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class FieldAttributeAppenderFactoryCompoundTest extends AbstractFieldAttributeAppenderTest {
 
@@ -14,8 +16,8 @@ public class FieldAttributeAppenderFactoryCompoundTest extends AbstractFieldAttr
     @Mock
     private FieldAttributeAppender first, second;
 
-    @Override
     @Before
+    @Override
     public void setUp() throws Exception {
         super.setUp();
         when(firstFactory.make(instrumentedType)).thenReturn(first);
@@ -30,6 +32,6 @@ public class FieldAttributeAppenderFactoryCompoundTest extends AbstractFieldAttr
         verifyNoMoreInteractions(first);
         verify(second).apply(fieldVisitor, fieldDescription, annotationValueFilter);
         verifyNoMoreInteractions(second);
-        verifyZeroInteractions(instrumentedType);
+        verifyNoMoreInteractions(instrumentedType);
     }
 }
