@@ -4,10 +4,9 @@ import org.junit.Before;
 import org.junit.Test;
 import org.mockito.Mock;
 
-import java.util.Collections;
-import java.util.List;
-
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class MethodAttributeAppenderFactoryCompoundTest extends AbstractMethodAttributeAppenderTest {
 
@@ -17,8 +16,8 @@ public class MethodAttributeAppenderFactoryCompoundTest extends AbstractMethodAt
     @Mock
     private MethodAttributeAppender first, second;
 
-    @Override
     @Before
+    @Override
     public void setUp() throws Exception {
         super.setUp();
         when(firstFactory.make(instrumentedType)).thenReturn(first);
@@ -33,6 +32,6 @@ public class MethodAttributeAppenderFactoryCompoundTest extends AbstractMethodAt
         verifyNoMoreInteractions(first);
         verify(second).apply(methodVisitor, methodDescription, annotationValueFilter);
         verifyNoMoreInteractions(second);
-        verifyZeroInteractions(instrumentedType);
+        verifyNoMoreInteractions(instrumentedType);
     }
 }
