@@ -1,5 +1,6 @@
 package net.bytebuddy.matcher;
 
+import net.bytebuddy.ClassFileVersion;
 import net.bytebuddy.description.ByteCodeElement;
 import net.bytebuddy.description.ModifierReviewable;
 import net.bytebuddy.description.NamedElement;
@@ -21,7 +22,11 @@ import java.io.Serializable;
 import java.lang.annotation.Inherited;
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
-import java.lang.reflect.*;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.net.URL;
 import java.net.URLClassLoader;
 import java.sql.SQLException;
@@ -35,16 +40,21 @@ import java.util.concurrent.ConcurrentMap;
 import static net.bytebuddy.matcher.ElementMatchers.isConstructor;
 import static net.bytebuddy.matcher.ElementMatchers.isToString;
 import static net.bytebuddy.matcher.ElementMatchers.named;
+import static net.bytebuddy.matcher.ElementMatchers.namedOneOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.fail;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.any;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
+import static org.mockito.Mockito.when;
 
 public class ElementMatchersTest {
 
     private static final String FOO = "foo", BAR = "bar", QUX = "qux";
 
-    private static final String SINGLE_DEFAULT_METHOD = "net.bytebuddy.test.precompiled.SingleDefaultMethodInterface";
+    private static final String SINGLE_DEFAULT_METHOD = "net.bytebuddy.test.precompiled.v8.SingleDefaultMethodInterface";
 
     @Rule
     public MethodRule javaVersionRule = new JavaVersionRule();
@@ -107,8 +117,8 @@ public class ElementMatchersTest {
 
     @Test
     public void testIsType() throws Exception {
-        assertThat(ElementMatchers.is(Object.class).matches(TypeDescription.OBJECT), is(true));
-        assertThat(ElementMatchers.is(String.class).matches(TypeDescription.OBJECT), is(false));
+        assertThat(ElementMatchers.is(Object.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
+        assertThat(ElementMatchers.is(String.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -220,9 +230,9 @@ public class ElementMatchersTest {
 
     @Test
     public void testAnyOfType() throws Exception {
-        assertThat(ElementMatchers.anyOf(Object.class).matches(TypeDescription.OBJECT), is(true));
-        assertThat(ElementMatchers.anyOf(String.class, Object.class).matches(TypeDescription.OBJECT), is(true));
-        assertThat(ElementMatchers.anyOf(String.class).matches(TypeDescription.OBJECT), is(false));
+        assertThat(ElementMatchers.anyOf(Object.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
+        assertThat(ElementMatchers.anyOf(String.class, Object.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
+        assertThat(ElementMatchers.anyOf(String.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -316,9 +326,9 @@ public class ElementMatchersTest {
 
     @Test
     public void testNoneOfType() throws Exception {
-        assertThat(ElementMatchers.noneOf(Object.class).matches(TypeDescription.OBJECT), is(false));
-        assertThat(ElementMatchers.noneOf(String.class, Object.class).matches(TypeDescription.OBJECT), is(false));
-        assertThat(ElementMatchers.noneOf(String.class).matches(TypeDescription.OBJECT), is(true));
+        assertThat(ElementMatchers.noneOf(Object.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+        assertThat(ElementMatchers.noneOf(String.class, Object.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+        assertThat(ElementMatchers.noneOf(String.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
     }
 
     @Test
@@ -380,18 +390,18 @@ public class ElementMatchersTest {
 
     @Test
     public void testAnyOf() throws Exception {
-        Object value = new Object(), otherValue = new Object();
-        assertThat(ElementMatchers.anyOf(value, otherValue).matches(value), is(true));
-        assertThat(ElementMatchers.anyOf(value, otherValue).matches(otherValue), is(true));
-        assertThat(ElementMatchers.anyOf(value, otherValue).matches(new Object()), is(false));
+        assertThat(ElementMatchers.anyOf(FOO, BAR).matches(FOO), is(true));
+        assertThat(ElementMatchers.anyOf(FOO, BAR).matches(BAR), is(true));
+        assertThat(ElementMatchers.anyOf(FOO, BAR).matches(new Object()), is(false));
+        assertThat(ElementMatchers.anyOf(FOO, BAR).toString(), is("(is(" + FOO + ") or is(" + BAR + "))"));
     }
 
     @Test
     public void testNoneOf() throws Exception {
-        Object value = new Object(), otherValue = new Object();
-        assertThat(ElementMatchers.noneOf(value, otherValue).matches(value), is(false));
-        assertThat(ElementMatchers.noneOf(value, otherValue).matches(otherValue), is(false));
-        assertThat(ElementMatchers.noneOf(value, otherValue).matches(new Object()), is(true));
+        assertThat(ElementMatchers.noneOf(FOO, BAR).matches(FOO), is(false));
+        assertThat(ElementMatchers.noneOf(FOO, BAR).matches(BAR), is(false));
+        assertThat(ElementMatchers.noneOf(FOO, BAR).matches(new Object()), is(true));
+        assertThat(ElementMatchers.noneOf(FOO, BAR).toString(), is("(not(is(" + FOO + ")) and not(is(" + BAR + ")))"));
     }
 
     @Test
@@ -424,17 +434,17 @@ public class ElementMatchersTest {
     public void testIsTypeVariable() throws Exception {
         assertThat(ElementMatchers.isVariable("T").matches(TypeDescription.ForLoadedType.of(GenericDeclaredBy.class).getTypeVariables().getOnly()), is(true));
         assertThat(ElementMatchers.isVariable(FOO).matches(TypeDescription.ForLoadedType.of(GenericDeclaredBy.class).getTypeVariables().getOnly()), is(false));
-        assertThat(ElementMatchers.isVariable(FOO).matches(TypeDescription.OBJECT), is(false));
+        assertThat(ElementMatchers.isVariable(FOO).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
     public void testMethodName() throws Exception {
         assertThat(ElementMatchers.hasMethodName(MethodDescription.TYPE_INITIALIZER_INTERNAL_NAME)
-                .matches(new MethodDescription.Latent.TypeInitializer(TypeDescription.OBJECT)), is(true));
+                .matches(new MethodDescription.Latent.TypeInitializer(TypeDescription.ForLoadedType.of(Object.class))), is(true));
         assertThat(ElementMatchers.hasMethodName(MethodDescription.CONSTRUCTOR_INTERNAL_NAME)
-                .matches(TypeDescription.OBJECT.getDeclaredMethods().filter(isConstructor()).getOnly()), is(true));
+                .matches(TypeDescription.ForLoadedType.of(Object.class).getDeclaredMethods().filter(isConstructor()).getOnly()), is(true));
         assertThat(ElementMatchers.hasMethodName("toString")
-                .matches(TypeDescription.OBJECT.getDeclaredMethods().filter(isToString()).getOnly()), is(true));
+                .matches(TypeDescription.ForLoadedType.of(Object.class).getDeclaredMethods().filter(isToString()).getOnly()), is(true));
     }
 
     @Test
@@ -446,6 +456,16 @@ public class ElementMatchersTest {
         assertThat(named(BAR).matches(byteCodeElement), is(false));
     }
 
+    @Test
+    public void testNamedOneOf() throws Exception {
+        ByteCodeElement byteCodeElement = mock(ByteCodeElement.class);
+        when(byteCodeElement.getActualName()).thenReturn(FOO);
+        assertThat(namedOneOf(FOO, BAR).matches(byteCodeElement), is(true));
+        assertThat(namedOneOf(FOO.toUpperCase(), BAR).matches(byteCodeElement), is(false));
+        assertThat(namedOneOf(FOO.toUpperCase()).matches(byteCodeElement), is(false));
+        assertThat(namedOneOf(BAR).matches(byteCodeElement), is(false));
+    }
+
     @Test
     public void testNamedIgnoreCase() throws Exception {
         ByteCodeElement byteCodeElement = mock(ByteCodeElement.class);
@@ -584,7 +604,7 @@ public class ElementMatchersTest {
         assertThat(ElementMatchers.isAnnotatedWith(IsAnnotatedWithAnnotation.class)
                 .matches(TypeDescription.ForLoadedType.of(IsAnnotatedWith.class)), is(true));
         assertThat(ElementMatchers.isAnnotatedWith(IsAnnotatedWithAnnotation.class)
-                .matches(TypeDescription.OBJECT), is(false));
+                .matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -805,6 +825,14 @@ public class ElementMatchersTest {
                 .matches(new MethodDescription.ForLoadedMethod(TakesArguments.class.getDeclaredMethod(BAR, String.class, int.class))), is(false));
     }
 
+    @Test
+    public void testTakesNoArguments() throws Exception {
+        assertThat(ElementMatchers.takesNoArguments()
+                .matches(new MethodDescription.ForLoadedMethod(TakesArguments.class.getDeclaredMethod(QUX))), is(true));
+        assertThat(ElementMatchers.takesNoArguments()
+                .matches(new MethodDescription.ForLoadedMethod(TakesArguments.class.getDeclaredMethod(FOO, Void.class))), is(false));
+    }
+
     @Test
     public void testDeclaresException() throws Exception {
         assertThat(ElementMatchers.declaresException(IOException.class)
@@ -915,7 +943,23 @@ public class ElementMatchersTest {
         assertThat(ElementMatchers.isVirtual().matches(new MethodDescription.ForLoadedMethod(IsVirtual.class.getDeclaredMethod("bar"))), is(false));
         assertThat(ElementMatchers.isVirtual().matches(new MethodDescription.ForLoadedMethod(IsVirtual.class.getDeclaredMethod("qux"))), is(true));
         assertThat(ElementMatchers.isVirtual().matches(new MethodDescription.ForLoadedConstructor(IsVirtual.class.getDeclaredConstructor())), is(false));
-        assertThat(ElementMatchers.isVirtual().matches(new MethodDescription.Latent.TypeInitializer(TypeDescription.OBJECT)), is(false));
+        assertThat(ElementMatchers.isVirtual().matches(new MethodDescription.Latent.TypeInitializer(TypeDescription.ForLoadedType.of(Object.class))), is(false));
+    }
+
+    @Test
+    public void testIsMain() throws Exception {
+        assertThat(ElementMatchers.isMain(false)
+                .matches(new MethodDescription.ForLoadedMethod(MainMethod.class.getDeclaredMethod("main", String[].class))), is(true));
+        assertThat(ElementMatchers.isMain()
+                .matches(new MethodDescription.ForLoadedMethod(MainMethod.class.getDeclaredMethod("main", String[].class))), is(true));
+        assertThat(ElementMatchers.isMain(false)
+                .matches(new MethodDescription.ForLoadedMethod(MainMethod.class.getDeclaredMethod("main"))), is(false));
+        assertThat(ElementMatchers.isMain()
+                .matches(new MethodDescription.ForLoadedMethod(MainMethod.class.getDeclaredMethod("main"))), is(true));
+        assertThat(ElementMatchers.isMain(false)
+                .matches(new MethodDescription.ForLoadedMethod(Object.class.getDeclaredMethod("toString"))), is(false));
+        assertThat(ElementMatchers.isMain()
+                .matches(new MethodDescription.ForLoadedMethod(Object.class.getDeclaredMethod("toString"))), is(false));
     }
 
     @Test
@@ -1104,7 +1148,7 @@ public class ElementMatchersTest {
 
     @Test
     public void testHasSignature() throws Exception {
-        MethodDescription.SignatureToken signatureToken = new MethodDescription.SignatureToken("toString", TypeDescription.STRING, Collections.<TypeDescription>emptyList());
+        MethodDescription.SignatureToken signatureToken = new MethodDescription.SignatureToken("toString", TypeDescription.ForLoadedType.of(String.class), Collections.<TypeDescription>emptyList());
         assertThat(ElementMatchers.hasSignature(signatureToken)
                 .matches(new MethodDescription.ForLoadedMethod(Object.class.getDeclaredMethod("toString"))), is(true));
         assertThat(ElementMatchers.hasSignature(signatureToken)
@@ -1113,20 +1157,28 @@ public class ElementMatchersTest {
 
     @Test
     public void testIsSubOrSuperType() throws Exception {
-        assertThat(ElementMatchers.isSubTypeOf(String.class).matches(TypeDescription.OBJECT), is(false));
-        assertThat(ElementMatchers.isSubTypeOf(Object.class).matches(TypeDescription.STRING), is(true));
-        assertThat(ElementMatchers.isSubTypeOf(Serializable.class).matches(TypeDescription.STRING), is(true));
-        assertThat(ElementMatchers.isSuperTypeOf(Object.class).matches(TypeDescription.STRING), is(false));
-        assertThat(ElementMatchers.isSuperTypeOf(String.class).matches(TypeDescription.OBJECT), is(true));
+        assertThat(ElementMatchers.isSubTypeOf(String.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+        assertThat(ElementMatchers.isSubTypeOf(Object.class).matches(TypeDescription.ForLoadedType.of(String.class)), is(true));
+        assertThat(ElementMatchers.isSubTypeOf(Serializable.class).matches(TypeDescription.ForLoadedType.of(String.class)), is(true));
+        assertThat(ElementMatchers.isSuperTypeOf(Object.class).matches(TypeDescription.ForLoadedType.of(String.class)), is(false));
+        assertThat(ElementMatchers.isSuperTypeOf(String.class).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
         assertThat(ElementMatchers.isSuperTypeOf(String.class).matches(TypeDescription.ForLoadedType.of(Serializable.class)), is(true));
     }
 
+    @Test
+    public void testHasSuperClass() throws Exception {
+        assertThat(ElementMatchers.hasSuperClass(ElementMatchers.is(Object.class)).matches(TypeDescription.ForLoadedType.of(String.class)), is(true));
+        assertThat(ElementMatchers.hasSuperClass(ElementMatchers.is(String.class)).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+        assertThat(ElementMatchers.hasSuperClass(ElementMatchers.is(Serializable.class)).matches(TypeDescription.ForLoadedType.of(String.class)), is(false));
+        assertThat(ElementMatchers.hasSuperClass(ElementMatchers.is(Serializable.class)).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+    }
+
     @Test
     public void testHasSuperType() throws Exception {
-        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Object.class)).matches(TypeDescription.STRING), is(true));
-        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(String.class)).matches(TypeDescription.OBJECT), is(false));
-        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Serializable.class)).matches(TypeDescription.STRING), is(true));
-        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Serializable.class)).matches(TypeDescription.OBJECT), is(false));
+        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Object.class)).matches(TypeDescription.ForLoadedType.of(String.class)), is(true));
+        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(String.class)).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Serializable.class)).matches(TypeDescription.ForLoadedType.of(String.class)), is(true));
+        assertThat(ElementMatchers.hasSuperType(ElementMatchers.is(Serializable.class)).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -1139,8 +1191,8 @@ public class ElementMatchersTest {
 
     @Test
     public void testTypeSort() throws Exception {
-        assertThat(ElementMatchers.ofSort(TypeDefinition.Sort.NON_GENERIC).matches(TypeDescription.OBJECT), is(true));
-        assertThat(ElementMatchers.ofSort(TypeDefinition.Sort.VARIABLE).matches(TypeDescription.OBJECT), is(false));
+        assertThat(ElementMatchers.ofSort(TypeDefinition.Sort.NON_GENERIC).matches(TypeDescription.ForLoadedType.of(Object.class)), is(true));
+        assertThat(ElementMatchers.ofSort(TypeDefinition.Sort.VARIABLE).matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -1148,11 +1200,11 @@ public class ElementMatchersTest {
         assertThat(ElementMatchers.declaresField(ElementMatchers.isAnnotatedWith(OtherAnnotation.class))
                 .matches(TypeDescription.ForLoadedType.of(DeclaresFieldOrMethod.class)), is(true));
         assertThat(ElementMatchers.declaresField(ElementMatchers.isAnnotatedWith(OtherAnnotation.class))
-                .matches(TypeDescription.OBJECT), is(false));
+                .matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
         assertThat(ElementMatchers.declaresMethod(ElementMatchers.isAnnotatedWith(OtherAnnotation.class))
                 .matches(TypeDescription.ForLoadedType.of(DeclaresFieldOrMethod.class)), is(true));
         assertThat(ElementMatchers.declaresMethod(ElementMatchers.isAnnotatedWith(OtherAnnotation.class))
-                .matches(TypeDescription.OBJECT), is(false));
+                .matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
     }
 
     @Test
@@ -1220,12 +1272,44 @@ public class ElementMatchersTest {
         assertThat(ElementMatchers.ofType(ElementMatchers.is(URLClassLoader.class)).matches(null), is(false));
     }
 
+    @Test
+    public void testIsPrimitive() {
+        assertThat(ElementMatchers.isPrimitive().matches(TypeDescription.ForLoadedType.of(void.class)), is(true));
+        assertThat(ElementMatchers.isPrimitive().matches(TypeDescription.ForLoadedType.of(int.class)), is(true));
+        assertThat(ElementMatchers.isPrimitive().matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+    }
+
+    @Test
+    public void testIsArray() {
+        assertThat(ElementMatchers.isArray().matches(TypeDescription.ForLoadedType.of(void.class)), is(false));
+        assertThat(ElementMatchers.isArray().matches(TypeDescription.ForLoadedType.of(int[].class)), is(true));
+        assertThat(ElementMatchers.isArray().matches(TypeDescription.ForLoadedType.of(Object.class)), is(false));
+    }
+
     @Test
     public void testSupportsModules() throws Exception {
         assertThat(ElementMatchers.supportsModules().matches(mock(JavaModule.class)), is(true));
         assertThat(ElementMatchers.supportsModules().matches(null), is(false));
     }
 
+    @Test
+    public void testClassFileVersionAtLeast() throws Exception {
+        TypeDescription typeDescription = mock(TypeDescription.class);
+        when(typeDescription.getClassFileVersion()).thenReturn(ClassFileVersion.JAVA_V6);
+        assertThat(ElementMatchers.hasClassFileVersionAtLeast(ClassFileVersion.JAVA_V5).matches(typeDescription), is(true));
+        assertThat(ElementMatchers.hasClassFileVersionAtLeast(ClassFileVersion.JAVA_V6).matches(typeDescription), is(true));
+        assertThat(ElementMatchers.hasClassFileVersionAtLeast(ClassFileVersion.JAVA_V7).matches(typeDescription), is(false));
+    }
+
+    @Test
+    public void testClassFileVersionAtMost() throws Exception {
+        TypeDescription typeDescription = mock(TypeDescription.class);
+        when(typeDescription.getClassFileVersion()).thenReturn(ClassFileVersion.JAVA_V6);
+        assertThat(ElementMatchers.hasClassFileVersionAtMost(ClassFileVersion.JAVA_V5).matches(typeDescription), is(false));
+        assertThat(ElementMatchers.hasClassFileVersionAtMost(ClassFileVersion.JAVA_V6).matches(typeDescription), is(true));
+        assertThat(ElementMatchers.hasClassFileVersionAtMost(ClassFileVersion.JAVA_V7).matches(typeDescription), is(true));
+    }
+
     @Test(expected = UnsupportedOperationException.class)
     public void testConstructorIsHidden() throws Exception {
         assertThat(Modifier.isPrivate(ElementMatchers.class.getDeclaredConstructor().getModifiers()), is(true));
@@ -1235,7 +1319,7 @@ public class ElementMatchersTest {
             constructor.newInstance();
             fail();
         } catch (InvocationTargetException exception) {
-            throw (UnsupportedOperationException) exception.getCause();
+            throw (UnsupportedOperationException) exception.getTargetException();
         }
     }
 
@@ -1318,9 +1402,12 @@ public class ElementMatchersTest {
     @SuppressWarnings("unused")
     private abstract static class TakesArguments {
 
+        abstract void qux();
+
         abstract void foo(Void a);
 
         abstract void bar(String a, int b);
+
     }
 
     private abstract static class CanThrow {
@@ -1338,7 +1425,6 @@ public class ElementMatchersTest {
 
         public static class Extension extends GenericType<Void> {
 
-            @Override
             public void foo(Void t) {
                 /* empty */
             }
@@ -1367,8 +1453,8 @@ public class ElementMatchersTest {
             return super.toString();
         }
 
-        @Override
-        @SuppressWarnings("deprecation")
+        //@Override
+        @SuppressWarnings({"deprecation", "removal"})
         protected void finalize() throws Throwable {
             super.finalize();
         }
@@ -1377,14 +1463,13 @@ public class ElementMatchersTest {
     private static class CloneMethods {
 
         @Override
-        public CloneMethods clone() throws CloneNotSupportedException {
+        public CloneMethods clone() {
             return new CloneMethods();
         }
 
         public Object clone(int someArgument) {
             return null;
         }
-
     }
 
     @SuppressWarnings("unused")
@@ -1508,6 +1593,17 @@ public class ElementMatchersTest {
         }
     }
 
+    private static class MainMethod {
+
+        void main() {
+            /* empty */
+        }
+
+        public static void main(String[] args) {
+            /* empty */
+        }
+    }
+
     @SuppressWarnings("unused")
     public static class GenericFieldType<T> {
 
