@@ -23,7 +23,10 @@ import java.util.List;
 
 import static net.bytebuddy.matcher.ElementMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.mockito.Mockito.*;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.verifyNoMoreInteractions;
 
 @RunWith(Parameterized.class)
 public class TypePoolDefaultWithLazyResolutionTypeDescriptionTest extends AbstractTypeDescriptionTest {
@@ -77,6 +80,19 @@ public class TypePoolDefaultWithLazyResolutionTypeDescriptionTest extends Abstra
         return describe(type).getInterfaces().get(index);
     }
 
+    @Test
+    public void testIllegalResolutionDoesNotPersist() throws Exception {
+        TypePool typePool = new TypePool.Default.WithLazyResolution(TypePool.CacheProvider.WithIllegalResolutionReattempt.of(new TypePool.CacheProvider.Simple()),
+                ClassFileLocator.NoOp.INSTANCE,
+                TypePool.Default.ReaderMode.FAST,
+                lazinessMode);
+        String name = "foo.Bar";
+        TypePool.Resolution resolution = typePool.describe(name);
+        assertThat(resolution.resolve().getName(), CoreMatchers.is(name));
+        assertThat(resolution.isResolved(), CoreMatchers.is(false));
+        assertThat(typePool.describe(name).resolve().getName(), CoreMatchers.is(name));
+    }
+
     @Test
     public void testTypeIsLazy() throws Exception {
         ClassFileLocator classFileLocator = spy(ClassFileLocator.ForClassLoader.ofSystemLoader());
